c     Copyright (C) 2008-2009 Vincent Eymet
c
c     Planet_EMC is free software; you can redistribute it and/or modify
c     it under the terms of the GNU General Public License as published by
c     the Free Software Foundation; either version 3, or (at your option)
c     any later version.

c     Planet_EMC is distributed in the hope that it will be useful,
c     but WITHOUT ANY WARRANTY; without even the implied warranty of
c     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c     GNU General Public License for more details.

c     You should have received a copy of the GNU General Public License
c     along with Planet_EMC; if not, see <http://www.gnu.org/licenses/>
c
      subroutine lin_interp1d(x1,x2,x,f1,f2,f)
      implicit none
c
c     Purpose: linear interpolation of quantity "f" between f1 and f2
c
c     Inputs:
c       + x1: absica where f1 is given
c       + x2: absica where f2 is given
c       + x : absicas where f is interpolated
c       + f1: quantity f defined at x1
c       + f2: quantity f defined at x2
c
c     Output: f is the quantity interpolated between f1 and f2
c
      double precision x1,x2,x,f1,f2,f

      f=f1+(f2-f1)/(x2-x1)*(x-x1)

      return
      end


      subroutine lin_interp2d(x1,y1,x2,y2,x3,y3,x,y,f1,f2,f3,f)
      implicit none
c
c     Purpose: linear interpolation of quantity "f" between
c     three points on a plane
c
c     Inputs:
c       + x1,y1: coordinates of point 1
c       + x2,y2: coordinates of point 2
c       + x3,y3: coordinates of point 3
c       + x,y: coordinates of the point quantity "f" has to be interpolated at
c       + f1: value of function f at point 1
c       + f2: value of function f at point 2
c       + f3: value of function f at point 3
c     
c     Output:
c       + f: value of function at point (x,y)
c
      integer strlen
      double precision x1,y1,x2,y2,x3,y3,x,y
      double precision f1,f2,f3,f
      double precision u1,u2,u3
      double precision v1,v2,v3
      character*100 label
      label='subroutine lin_interp2d'

      u1=x2-x1
      u2=y2-y1
      u3=f2-f1
      v1=x3-x1
      v2=y3-y1
      v3=f3-f1

      if (u1*v2-u2*v1.eq.0.0D+0) then
         write(*,*) 'Error from ',label(1:strlen(label)),' :'
         write(*,*) 'could not compute equation of the plane'
         stop
      endif
      f=f1+((u2*v3-u3*v2)*(x1-x)+(u3*v1-u1*v3)*(y1-y))/(u1*v2-u2*v1)

      return
      end






      subroutine lin_interp2d_spher(dim,n,Pi,fi,P,d,f_out)
      implicit none
      include 'max.inc'
c
c     Purpose: linear interpolation of quantity "f" between a number of
c     2D observations within a spherical coordinates system
c
c     Inputs:
c       + dim: dimension of vectors
c       + n: number of observed points
c       + Pi: (theta,phi) latitude and longitude of the n observed points
c       + fi: list of the n observed values of function f (for instance, altitude)
c       + P: (theta,phi) latitude and longitude of the point function f
c               has to be interpolated at
c       + d: positive integer value (default: 1)
c
c     Outputs:
c       + f_out: value of function f at position x
c

c     input
      integer dim,n
      double precision Pi(1:2,1:Nobs_mx)
      double precision fi(1:Nobs_mx)
      double precision P(1:2)
      integer d
c     output
      double precision f_out
c     temp
      integer mod
      double precision Pdata(1:2)
      double precision a(1:Nobs_mx)
      double precision lambda(1:Nobs_mx)
      integer i
      double precision di,sum_ai
      integer strlen
      character*100 label
      label='subroutine lin_interp2d_spher'

      if (n.gt.Nobs_mx) then
         write(*,*) 'Error from ',label(1:strlen(label)),' :'
         write(*,*) 'n=',n
         write(*,*) 'while Nobs_mx=',Nobs_mx
         stop
      endif

c     if all values "fi" are identical, then "f_out" is known !
      call any_difference(n,fi,mod)
      if (mod.eq.0) then
         f_out=fi(1)
         goto 123
      endif

c     if values "fi" are different, then perform a real interpolation
      sum_ai=0.0D+0
      do i=1,n
         Pdata(1)=Pi(1,i)
         Pdata(2)=Pi(2,i)
         call distance_on_sphere(dim,Pdata,P,fi(i),di)
         if (di.eq.0.0D+0) then
            f_out=fi(i)
            goto 123
         else
            a(i)=1.0D+0/di**d
            sum_ai=sum_ai+a(i)
         endif
      enddo
      do i=1,n
         lambda(i)=a(i)/sum_ai
      enddo

      f_out=0.0D+0
      do i=1,n
         f_out=f_out+fi(i)*lambda(i)
      enddo

 123  continue
      
      return
      end



      subroutine lin_interp3d(n,xi,fi,x,d,f_out)
      implicit none
      include 'max.inc'
c
c     Purpose: linear interpolation of quantity "f" between a number of
c     3D observations
c
c     Inputs:
c       + n: number of observed points
c       + xi: (x,y,z) 3D cartesian coordinates of the n observed points
c       + fi: list of the n observed values of function f
c       + x: (x,y,z) 3D cartesian coordinates of the point function f
c               has to be interpolated at
c       + d: positive integer value (default: 1)
c
c     Outputs:
c       + f_out: value of function f at position x
c

c     input
      integer n
      double precision xi(1:3,1:Nobs_mx)
      double precision fi(1:Nobs_mx)
      double precision x(1:3)
      integer d
c     output
      double precision f_out
c     temp
      integer mod
      double precision a(1:Nobs_mx)
      double precision lambda(1:Nobs_mx)
      integer i
      double precision di,sum_ai
      integer strlen
      character*100 label
      label='subroutine lin_interp3d'

      if (n.gt.Nobs_mx) then
         write(*,*) 'Error from ',label(1:strlen(label)),' :'
         write(*,*) 'n=',n
         write(*,*) 'while Nobs_mx=',Nobs_mx
         stop
      endif

c     if all values "fi" are identical, then "f_out" is known !
      call any_difference(n,fi,mod)
      if (mod.eq.0) then
         f_out=fi(1)
         goto 123
      endif

      sum_ai=0.0D+0
      do i=1,n
         call distance_cart(xi(1,i),xi(2,i),xi(3,i),x(1),x(2),x(3),di)
         if (di.eq.0.0D+0) then
            f_out=fi(i)
            goto 123
         else
            a(i)=1.0D+0/di**d
            sum_ai=sum_ai+a(i)
         endif
      enddo
      do i=1,n
         lambda(i)=a(i)/sum_ai
      enddo

      f_out=0.0D+0
      do i=1,n
         f_out=f_out+fi(i)*lambda(i)
      enddo

 123  continue
      
      return
      end



      subroutine any_difference(n,values,diff)
      implicit none
      include 'max.inc'
c     
c     Purpose: to check whether there are differences or not
c     between the provided "n" values
c
c     Inputs:
c       + n: number of values
c       + values: the "n" values
c     
c     Output:
c       + diff:
c         - diff=0: no differences between the "n" values
c         - diff=1: there are at least 2 different values
c

c     inputs
      integer n
      double precision values(1:Nobs_mx)
c     output
      integer diff
c     temp
      integer strlen,i
      character*100 label
      label='subroutine any_difference'

      if (n.gt.Nobs_mx) then
         write(*,*) 'Error from ',label(1:strlen(label)),' :'
         write(*,*) 'n=',n
         write(*,*) 'while Nobs_mx=',Nobs_mx
         stop
      endif

      diff=0
      do i=2,n
         if (values(i).ne.values(1)) then
            diff=1
            goto 111
         endif
      enddo  ! i
 111  continue

      return
      end





      subroutine gas_temp(dim,Nr,Ntheta,Nphi,ir,itheta,iphi,r,theta,phi,
     &     r_nodes,r_south_pole,r_north_pole,ca,cb,cc,
     &     xr,xtheta,xphi,T,r_mc,Tgas)
      implicit none
      include 'max.inc'
c
c     Purpose: to interpolate the temperature within a given cell
c
c     Inputs:
c       + Nr,Ntheta,Nphi: number of cells
c       + ir,itheta,iphi: indexes of the cell
c       + r,theta,phi: limits of spherical cells
c       + xr,xtheta,xphi: coordinates of the point temperature has to be interpolated at
c       + T: temperature points
c       + r_mc: radiuses at cells mass center
c
c     Outputs:
c       + Tgas: value of the temperature at point (r,theta,phi)
c

c     inputs
      integer dim
      integer Nr(1:Ntheta_mx,1:Nphi_mx),Ntheta,Nphi
      integer ir,itheta,iphi
      double precision r(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision theta(0:Ntheta_mx)
      double precision phi(0:Nphi_mx)
      double precision r_nodes(1:Ntheta_mx,1:Nphi_mx,1:4)
      double precision r_south_pole
      double precision r_north_pole
      double precision ca(1:4)
      double precision cb(1:4)
      double precision cc(1:4)
      double precision xr,xtheta,xphi
      double precision T(0:Nz_mx+1,1:Ntheta_mx,1:Nphi_mx)
      double precision r_mc(1:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
c     output
      double precision Tgas
c     temp
      integer strlen,i
      integer ir_prev,ir_next,ir_tmp
      integer itheta_prev,itheta_next
      integer iphi_prev,iphi_next,iphi_mod
      integer n,d,in
      double precision xi(1:3,1:Nobs_mx)
      double precision x(1:3)
      double precision coord(1:Nmat_mx)
      double precision fi(1:Nobs_mx)
      double precision rtmp_prev,rtmp,rtmp_next
      double precision raf1,raf2
      double precision Tmin,Tmax,dT
      character*100 label
      label='subroutine gas_temp'
      dT=1.0D-1

      coord(1)=xr
      coord(2)=xtheta
      coord(3)=xphi
      call is_in_box(coord,r(ir-1,itheta,iphi),r(ir,itheta,iphi),
     &     theta(itheta-1),theta(itheta),
     &     phi(iphi-1),phi(iphi),in)
      if (in.eq.0) then
         write(*,*) 'Error from ',label(1:strlen(label)),' :'
         write(*,*) 'The provided position is out of cell;'
         write(*,*) 'Coordinates of the point:'
         write(*,*) 'r=',xr
         write(*,*) 'theta=',xtheta
         write(*,*) 'phi=',xphi
         write(*,*) 'Limits of the cell index:',ir,',',itheta,',',iphi
         write(*,*) 'r=',r(ir-1,itheta,iphi),',',r(ir,itheta,iphi)
         write(*,*) 'theta=',theta(itheta-1),',',theta(itheta)
         write(*,*) 'phi=',phi(iphi-1),',',phi(iphi)
         stop
      endif

      if (ir.gt.1) then
         call prev_r_cell(Nr,ir,itheta,iphi,ir_prev)
      endif
      if (ir.lt.Nr(itheta,iphi)) then
         call next_r_cell(Nr,ir,itheta,iphi,ir_next)
      endif
      call prev_theta_cell(Ntheta,Nphi,itheta,iphi,
     &     itheta_prev,iphi_mod)
      call next_theta_cell(Ntheta,Nphi,itheta,iphi,
     &     itheta_next,iphi_mod)
      call prev_phi_cell(Nphi,iphi,iphi_prev)
      call next_phi_cell(Nphi,iphi,iphi_next)

c     number of observed temperatures
      n=27
c     set cartesian coordinates of the n observation positions
c     points 1-9
      call ir_tmp_cells(dim,Nr,Ntheta,Nphi,r,theta,phi,
     &     r_nodes,r_south_pole,r_north_pole,ca,cb,cc,
     &     r_mc,T,coord,itheta_prev,iphi_next,
     &     rtmp_prev,rtmp,rtmp_next,fi(1),raf1,raf2)
c      call ir_tmp_cells(Nr,r,r_mc,T,xr,itheta_prev,iphi_next,
c     &     rtmp_prev,rtmp,rtmp_next,fi(1),raf1,raf2)
      call spher2cart(label,1,rtmp_prev,
     &     (theta(itheta_prev-1)+theta(itheta_prev))/2.0D+0,
     &     (phi(iphi_next-1)+phi(iphi_next))/2.0D+0,
     &     xi(1,1),xi(2,1),xi(3,1))
      call ir_tmp_cells(dim,Nr,Ntheta,Nphi,r,theta,phi,
     &     r_nodes,r_south_pole,r_north_pole,ca,cb,cc,
     &     r_mc,T,coord,itheta_prev,iphi,
     &     rtmp_prev,rtmp,rtmp_next,fi(2),raf1,raf2)
c      call ir_tmp_cells(Nr,r,r_mc,T,xr,itheta_prev,iphi,
c     &     rtmp_prev,rtmp,rtmp_next,fi(2),raf1,raf2)
      call spher2cart(label,1,rtmp_prev,
     &     (theta(itheta_prev-1)+theta(itheta_prev))/2.0D+0,
     &     (phi(iphi-1)+phi(iphi))/2.0D+0,
     &     xi(1,2),xi(2,2),xi(3,2))
      call ir_tmp_cells(dim,Nr,Ntheta,Nphi,r,theta,phi,
     &     r_nodes,r_south_pole,r_north_pole,ca,cb,cc,
     &     r_mc,T,coord,itheta_prev,iphi_prev,
     &     rtmp_prev,rtmp,rtmp_next,fi(3),raf1,raf2)
c      call ir_tmp_cells(Nr,r,r_mc,T,xr,itheta_prev,iphi_prev,
c     &     rtmp_prev,rtmp,rtmp_next,fi(3),raf1,raf2)
      call spher2cart(label,1,rtmp_prev,
     &     (theta(itheta_prev-1)+theta(itheta_prev))/2.0D+0,
     &     (phi(iphi_prev-1)+phi(iphi_prev))/2.0D+0,
     &     xi(1,3),xi(2,3),xi(3,3))
      call ir_tmp_cells(dim,Nr,Ntheta,Nphi,r,theta,phi,
     &     r_nodes,r_south_pole,r_north_pole,ca,cb,cc,
     &     r_mc,T,coord,itheta,iphi_next,
     &     rtmp_prev,rtmp,rtmp_next,fi(4),raf1,raf2)
c      call ir_tmp_cells(Nr,r,r_mc,T,xr,itheta,iphi_next,
c     &     rtmp_prev,rtmp,rtmp_next,fi(4),raf1,raf2)
      call spher2cart(label,1,rtmp_prev,
     &     (theta(itheta-1)+theta(itheta))/2.0D+0,
     &     (phi(iphi_next-1)+phi(iphi_next))/2.0D+0,
     &     xi(1,4),xi(2,4),xi(3,4))
      call ir_tmp_cells(dim,Nr,Ntheta,Nphi,r,theta,phi,
     &     r_nodes,r_south_pole,r_north_pole,ca,cb,cc,
     &     r_mc,T,coord,itheta,iphi,
     &     rtmp_prev,rtmp,rtmp_next,fi(5),raf1,raf2)
c      call ir_tmp_cells(Nr,r,r_mc,T,xr,itheta,iphi,
c     &     rtmp_prev,rtmp,rtmp_next,fi(5),raf1,raf2)
      call spher2cart(label,1,rtmp_prev,
     &     (theta(itheta-1)+theta(itheta))/2.0D+0,
     &     (phi(iphi-1)+phi(iphi))/2.0D+0,
     &     xi(1,5),xi(2,5),xi(3,5))
      call ir_tmp_cells(dim,Nr,Ntheta,Nphi,r,theta,phi,
     &     r_nodes,r_south_pole,r_north_pole,ca,cb,cc,
     &     r_mc,T,coord,itheta,iphi_prev,
     &     rtmp_prev,rtmp,rtmp_next,fi(6),raf1,raf2)
c      call ir_tmp_cells(Nr,r,r_mc,T,xr,itheta,iphi_prev,
c     &     rtmp_prev,rtmp,rtmp_next,fi(6),raf1,raf2)
      call spher2cart(label,1,rtmp_prev,
     &     (theta(itheta-1)+theta(itheta))/2.0D+0,
     &     (phi(iphi_prev-1)+phi(iphi_prev))/2.0D+0,
     &     xi(1,6),xi(2,6),xi(3,6))
      call ir_tmp_cells(dim,Nr,Ntheta,Nphi,r,theta,phi,
     &     r_nodes,r_south_pole,r_north_pole,ca,cb,cc,
     &     r_mc,T,coord,itheta_next,iphi_next,
     &     rtmp_prev,rtmp,rtmp_next,fi(7),raf1,raf2)
c      call ir_tmp_cells(Nr,r,r_mc,T,xr,itheta_next,iphi_next,
c     &     rtmp_prev,rtmp,rtmp_next,fi(7),raf1,raf2)
      call spher2cart(label,1,rtmp_prev,
     &     (theta(itheta_next-1)+theta(itheta_next))/2.0D+0,
     &     (phi(iphi_next-1)+phi(iphi_next))/2.0D+0,
     &     xi(1,7),xi(2,7),xi(3,7))
      call ir_tmp_cells(dim,Nr,Ntheta,Nphi,r,theta,phi,
     &     r_nodes,r_south_pole,r_north_pole,ca,cb,cc,
     &     r_mc,T,coord,itheta_next,iphi,
     &     rtmp_prev,rtmp,rtmp_next,fi(8),raf1,raf2)
c      call ir_tmp_cells(Nr,r,r_mc,T,xr,itheta_next,iphi,
c     &     rtmp_prev,rtmp,rtmp_next,fi(8),raf1,raf2)
      call spher2cart(label,1,rtmp_prev,
     &     (theta(itheta_next-1)+theta(itheta_next))/2.0D+0,
     &     (phi(iphi-1)+phi(iphi))/2.0D+0,
     &     xi(1,8),xi(2,8),xi(3,8))
      call ir_tmp_cells(dim,Nr,Ntheta,Nphi,r,theta,phi,
     &     r_nodes,r_south_pole,r_north_pole,ca,cb,cc,
     &     r_mc,T,coord,itheta_next,iphi_prev,
     &     rtmp_prev,rtmp,rtmp_next,fi(9),raf1,raf2)
c      call ir_tmp_cells(Nr,r,r_mc,T,xr,itheta_next,iphi_prev,
c     &     rtmp_prev,rtmp,rtmp_next,fi(9),raf1,raf2)
      call spher2cart(label,1,rtmp_prev,
     &     (theta(itheta_next-1)+theta(itheta_next))/2.0D+0,
     &     (phi(iphi_prev-1)+phi(iphi_prev))/2.0D+0,
     &     xi(1,9),xi(2,9),xi(3,9))
c     points 10-18
      call ir_tmp_cells(dim,Nr,Ntheta,Nphi,r,theta,phi,
     &     r_nodes,r_south_pole,r_north_pole,ca,cb,cc,
     &     r_mc,T,coord,itheta_prev,iphi_next,
     &     rtmp_prev,rtmp,rtmp_next,fi(10),raf1,raf2)
c      call ir_tmp_cells(Nr,r,r_mc,T,xr,itheta_prev,iphi_next,
c     &     rtmp_prev,rtmp,rtmp_next,raf1,fi(10),raf2)
      call spher2cart(label,1,rtmp,
     &     (theta(itheta_prev-1)+theta(itheta_prev))/2.0D+0,
     &     (phi(iphi_next-1)+phi(iphi_next))/2.0D+0,
     &     xi(1,10),xi(2,10),xi(3,10))
      call ir_tmp_cells(dim,Nr,Ntheta,Nphi,r,theta,phi,
     &     r_nodes,r_south_pole,r_north_pole,ca,cb,cc,
     &     r_mc,T,coord,itheta_prev,iphi,
     &     rtmp_prev,rtmp,rtmp_next,fi(11),raf1,raf2)
c      call ir_tmp_cells(Nr,r,r_mc,T,xr,itheta_prev,iphi,
c     &     rtmp_prev,rtmp,rtmp_next,raf1,fi(11),raf2)
      call spher2cart(label,1,rtmp,
     &     (theta(itheta_prev-1)+theta(itheta_prev))/2.0D+0,
     &     (phi(iphi-1)+phi(iphi))/2.0D+0,
     &     xi(1,11),xi(2,11),xi(3,11))
      call ir_tmp_cells(dim,Nr,Ntheta,Nphi,r,theta,phi,
     &     r_nodes,r_south_pole,r_north_pole,ca,cb,cc,
     &     r_mc,T,coord,itheta_prev,iphi_prev,
     &     rtmp_prev,rtmp,rtmp_next,fi(12),raf1,raf2)
c      call ir_tmp_cells(Nr,r,r_mc,T,xr,itheta_prev,iphi_prev,
c     &     rtmp_prev,rtmp,rtmp_next,raf1,fi(12),raf2)
      call spher2cart(label,1,rtmp,
     &     (theta(itheta_prev-1)+theta(itheta_prev))/2.0D+0,
     &     (phi(iphi_prev-1)+phi(iphi_prev))/2.0D+0,
     &     xi(1,12),xi(2,12),xi(3,12))
      call ir_tmp_cells(dim,Nr,Ntheta,Nphi,r,theta,phi,
     &     r_nodes,r_south_pole,r_north_pole,ca,cb,cc,
     &     r_mc,T,coord,itheta,iphi_next,
     &     rtmp_prev,rtmp,rtmp_next,fi(13),raf1,raf2)
c      call ir_tmp_cells(Nr,r,r_mc,T,xr,itheta,iphi_next,
c     &     rtmp_prev,rtmp,rtmp_next,raf1,fi(13),raf2)
      call spher2cart(label,1,rtmp,
     &     (theta(itheta-1)+theta(itheta))/2.0D+0,
     &     (phi(iphi_next-1)+phi(iphi_next))/2.0D+0,
     &     xi(1,13),xi(2,13),xi(3,13))
      call ir_tmp_cells(dim,Nr,Ntheta,Nphi,r,theta,phi,
     &     r_nodes,r_south_pole,r_north_pole,ca,cb,cc,
     &     r_mc,T,coord,itheta,iphi,
     &     rtmp_prev,rtmp,rtmp_next,fi(14),raf1,raf2)
c      call ir_tmp_cells(Nr,r,r_mc,T,xr,itheta,iphi,
c     &     rtmp_prev,rtmp,rtmp_next,raf1,fi(14),raf2)
      call spher2cart(label,1,rtmp,
     &     (theta(itheta-1)+theta(itheta))/2.0D+0,
     &     (phi(iphi-1)+phi(iphi))/2.0D+0,
     &     xi(1,14),xi(2,14),xi(3,14))
      call ir_tmp_cells(dim,Nr,Ntheta,Nphi,r,theta,phi,
     &     r_nodes,r_south_pole,r_north_pole,ca,cb,cc,
     &     r_mc,T,coord,itheta,iphi_prev,
     &     rtmp_prev,rtmp,rtmp_next,fi(15),raf1,raf2)
c      call ir_tmp_cells(Nr,r,r_mc,T,xr,itheta,iphi_prev,
c     &     rtmp_prev,rtmp,rtmp_next,raf1,fi(15),raf2)
      call spher2cart(label,1,rtmp,
     &     (theta(itheta-1)+theta(itheta))/2.0D+0,
     &     (phi(iphi_prev-1)+phi(iphi_prev))/2.0D+0,
     &     xi(1,15),xi(2,15),xi(3,15))
      call ir_tmp_cells(dim,Nr,Ntheta,Nphi,r,theta,phi,
     &     r_nodes,r_south_pole,r_north_pole,ca,cb,cc,
     &     r_mc,T,coord,itheta_next,iphi_next,
     &     rtmp_prev,rtmp,rtmp_next,fi(16),raf1,raf2)
c      call ir_tmp_cells(Nr,r,r_mc,T,xr,itheta_next,iphi_next,
c     &     rtmp_prev,rtmp,rtmp_next,raf1,fi(16),raf2)
      call spher2cart(label,1,rtmp,
     &     (theta(itheta_next-1)+theta(itheta_next))/2.0D+0,
     &     (phi(iphi_next-1)+phi(iphi_next))/2.0D+0,
     &     xi(1,16),xi(2,16),xi(3,16))
      call ir_tmp_cells(dim,Nr,Ntheta,Nphi,r,theta,phi,
     &     r_nodes,r_south_pole,r_north_pole,ca,cb,cc,
     &     r_mc,T,coord,itheta_next,iphi,
     &     rtmp_prev,rtmp,rtmp_next,fi(17),raf1,raf2)
c      call ir_tmp_cells(Nr,r,r_mc,T,xr,itheta_next,iphi,
c     &     rtmp_prev,rtmp,rtmp_next,raf1,fi(17),raf2)
      call spher2cart(label,1,rtmp,
     &     (theta(itheta_next-1)+theta(itheta_next))/2.0D+0,
     &     (phi(iphi-1)+phi(iphi))/2.0D+0,
     &     xi(1,17),xi(2,17),xi(3,17))
      call ir_tmp_cells(dim,Nr,Ntheta,Nphi,r,theta,phi,
     &     r_nodes,r_south_pole,r_north_pole,ca,cb,cc,
     &     r_mc,T,coord,itheta_next,iphi_prev,
     &     rtmp_prev,rtmp,rtmp_next,fi(18),raf1,raf2)
c      call ir_tmp_cells(Nr,r,r_mc,T,xr,itheta_next,iphi_prev,
c     &     rtmp_prev,rtmp,rtmp_next,raf1,fi(18),raf2)
      call spher2cart(label,1,rtmp,
     &     (theta(itheta_next-1)+theta(itheta_next))/2.0D+0,
     &     (phi(iphi_prev-1)+phi(iphi_prev))/2.0D+0,
     &     xi(1,18),xi(2,18),xi(3,18))
c     points 19-27
      call ir_tmp_cells(dim,Nr,Ntheta,Nphi,r,theta,phi,
     &     r_nodes,r_south_pole,r_north_pole,ca,cb,cc,
     &     r_mc,T,coord,itheta_prev,iphi_next,
     &     rtmp_prev,rtmp,rtmp_next,fi(19),raf1,raf2)
c      call ir_tmp_cells(Nr,r,r_mc,T,xr,itheta_prev,iphi_next,
c     &     rtmp_prev,rtmp,rtmp_next,raf1,raf2,fi(19))
      call spher2cart(label,1,rtmp_next,
     &     (theta(itheta_prev-1)+theta(itheta_prev))/2.0D+0,
     &     (phi(iphi_next-1)+phi(iphi_next))/2.0D+0,
     &     xi(1,19),xi(2,19),xi(3,19))
      call ir_tmp_cells(dim,Nr,Ntheta,Nphi,r,theta,phi,
     &     r_nodes,r_south_pole,r_north_pole,ca,cb,cc,
     &     r_mc,T,coord,itheta_prev,iphi,
     &     rtmp_prev,rtmp,rtmp_next,fi(20),raf1,raf2)
c      call ir_tmp_cells(Nr,r,r_mc,T,xr,itheta_prev,iphi,
c     &     rtmp_prev,rtmp,rtmp_next,raf1,raf2,fi(20))
      call spher2cart(label,1,rtmp_next,
     &     (theta(itheta_prev-1)+theta(itheta_prev))/2.0D+0,
     &     (phi(iphi-1)+phi(iphi))/2.0D+0,
     &     xi(1,20),xi(2,20),xi(3,20))
      call ir_tmp_cells(dim,Nr,Ntheta,Nphi,r,theta,phi,
     &     r_nodes,r_south_pole,r_north_pole,ca,cb,cc,
     &     r_mc,T,coord,itheta_prev,iphi_prev,
     &     rtmp_prev,rtmp,rtmp_next,fi(21),raf1,raf2)
c      call ir_tmp_cells(Nr,r,r_mc,T,xr,itheta_prev,iphi_prev,
c     &     rtmp_prev,rtmp,rtmp_next,raf1,raf2,fi(21))
      call spher2cart(label,1,rtmp_next,
     &     (theta(itheta_prev-1)+theta(itheta_prev))/2.0D+0,
     &     (phi(iphi_prev-1)+phi(iphi_prev))/2.0D+0,
     &     xi(1,21),xi(2,21),xi(3,21))
      call ir_tmp_cells(dim,Nr,Ntheta,Nphi,r,theta,phi,
     &     r_nodes,r_south_pole,r_north_pole,ca,cb,cc,
     &     r_mc,T,coord,itheta,iphi_next,
     &     rtmp_prev,rtmp,rtmp_next,fi(22),raf1,raf2)
c      call ir_tmp_cells(Nr,r,r_mc,T,xr,itheta,iphi_next,
c     &     rtmp_prev,rtmp,rtmp_next,raf1,raf2,fi(22))
      call spher2cart(label,1,rtmp_next,
     &     (theta(itheta-1)+theta(itheta))/2.0D+0,
     &     (phi(iphi_next-1)+phi(iphi_next))/2.0D+0,
     &     xi(1,22),xi(2,22),xi(3,22))
      call ir_tmp_cells(dim,Nr,Ntheta,Nphi,r,theta,phi,
     &     r_nodes,r_south_pole,r_north_pole,ca,cb,cc,
     &     r_mc,T,coord,itheta,iphi,
     &     rtmp_prev,rtmp,rtmp_next,fi(23),raf1,raf2)
c      call ir_tmp_cells(Nr,r,r_mc,T,xr,itheta,iphi,
c     &     rtmp_prev,rtmp,rtmp_next,raf1,raf2,fi(23))
      call spher2cart(label,1,rtmp_next,
     &     (theta(itheta-1)+theta(itheta))/2.0D+0,
     &     (phi(iphi-1)+phi(iphi))/2.0D+0,
     &     xi(1,23),xi(2,23),xi(3,23))
      call ir_tmp_cells(dim,Nr,Ntheta,Nphi,r,theta,phi,
     &     r_nodes,r_south_pole,r_north_pole,ca,cb,cc,
     &     r_mc,T,coord,itheta,iphi_prev,
     &     rtmp_prev,rtmp,rtmp_next,fi(24),raf1,raf2)
c      call ir_tmp_cells(Nr,r,r_mc,T,xr,itheta,iphi_prev,
c     &     rtmp_prev,rtmp,rtmp_next,raf1,raf2,fi(24))
      call spher2cart(label,1,rtmp_next,
     &     (theta(itheta-1)+theta(itheta))/2.0D+0,
     &     (phi(iphi_prev-1)+phi(iphi_prev))/2.0D+0,
     &     xi(1,24),xi(2,24),xi(3,24))
      call ir_tmp_cells(dim,Nr,Ntheta,Nphi,r,theta,phi,
     &     r_nodes,r_south_pole,r_north_pole,ca,cb,cc,
     &     r_mc,T,coord,itheta_next,iphi_next,
     &     rtmp_prev,rtmp,rtmp_next,fi(25),raf1,raf2)
c      call ir_tmp_cells(Nr,r,r_mc,T,xr,itheta_next,iphi_next,
c     &     rtmp_prev,rtmp,rtmp_next,raf1,raf2,fi(25))
      call spher2cart(label,1,rtmp_next,
     &     (theta(itheta_next-1)+theta(itheta_next))/2.0D+0,
     &     (phi(iphi_next-1)+phi(iphi_next))/2.0D+0,
     &     xi(1,25),xi(2,25),xi(3,25))
      call ir_tmp_cells(dim,Nr,Ntheta,Nphi,r,theta,phi,
     &     r_nodes,r_south_pole,r_north_pole,ca,cb,cc,
     &     r_mc,T,coord,itheta_next,iphi,
     &     rtmp_prev,rtmp,rtmp_next,fi(26),raf1,raf2)
c      call ir_tmp_cells(Nr,r,r_mc,T,xr,itheta_next,iphi,
c     &     rtmp_prev,rtmp,rtmp_next,raf1,raf2,fi(26))
      call spher2cart(label,1,rtmp_next,
     &     (theta(itheta_next-1)+theta(itheta_next))/2.0D+0,
     &     (phi(iphi-1)+phi(iphi))/2.0D+0,
     &     xi(1,26),xi(2,26),xi(3,26))
      call ir_tmp_cells(dim,Nr,Ntheta,Nphi,r,theta,phi,
     &     r_nodes,r_south_pole,r_north_pole,ca,cb,cc,
     &     r_mc,T,coord,itheta_next,iphi_prev,
     &     rtmp_prev,rtmp,rtmp_next,fi(27),raf1,raf2)
c      call ir_tmp_cells(Nr,r,r_mc,T,xr,itheta_next,iphi_prev,
c     &     rtmp_prev,rtmp,rtmp_next,raf1,raf2,fi(27))
      call spher2cart(label,1,rtmp_next,
     &     (theta(itheta_next-1)+theta(itheta_next))/2.0D+0,
     &     (phi(iphi_prev-1)+phi(iphi_prev))/2.0D+0,
     &     xi(1,27),xi(2,27),xi(3,27))

c     looking for min and max values of "fi"
      Tmin=fi(1)
      Tmax=0.0D+0
      do i=1,n
         if (fi(i).lt.Tmin) then
            Tmin=fi(i)
         endif
         if (fi(i).gt.Tmax) then
            Tmax=fi(i)
         endif
      enddo

c     set cartesian coordinates of the point Tgas has to be interpolated at
      call spher2cart(label,1,xr,xtheta,xphi,x(1),x(2),x(3))
c     3D interpolation
      d=1
      call lin_interp3d(n,xi,fi,x,d,Tgas)
c     Debug
      if ((Tgas.lt.Tmin-dT).or.(Tgas.gt.Tmax+dT)) then
         write(*,*) 'Error from ',label(1:strlen(label)),' :'
         write(*,*) 'Tmin=',Tmin
         write(*,*) 'Tmax=',Tmax
         write(*,*) 'Tgas=',Tgas
         stop
      endif
c     Debug

      return
      end




      subroutine temp_interp_old(ir,itheta,iphi,r,theta,phi,
     &     xr,xtheta,xphi,Tcorner,T)
      implicit none
      include 'max.inc'
c
c     Purpose: to interpolate the temperature within a given cell
c
c     Inputs:
c       + ir,itheta,iphi: indexes of the cell
c       + r,theta,phi: limits of spherical cells
c       + xr,xtheta,xphi: coordinates of the point temperature has to be interpolated at
c       + Tcorner: temperatures of corners for each cell
c
c     Outputs:
c       + T: value of the temperature at point (r,theta,phi)
c

c     inputs
      integer ir,itheta,iphi
      double precision r(0:Nz_mx)
      double precision theta(0:Ntheta_mx)
      double precision phi(0:Nphi_mx)
      double precision xr,xtheta,xphi
      double precision Tcorner(1:Nz_mx,1:Ntheta_mx,1:Nphi_mx,1:8)
c     output
      double precision T
c     temp
      double precision dtheta,dphi
      double precision ctheta,cphi
      double precision Tp1,Tp2
      double precision x1,y1,x2,y2,x3,y3,f1,f2,f3

      dtheta=theta(itheta)-theta(itheta-1)
      dphi=phi(iphi)-phi(iphi-1)
      ctheta=(xtheta-theta(itheta-1))/dtheta
      cphi=(xphi-phi(iphi-1))/dphi

      x1=theta(itheta-1)
      y1=phi(iphi-1)
      f1=Tcorner(ir,itheta,iphi,2)
      x3=theta(itheta)
      y3=phi(iphi)
      f3=Tcorner(ir,itheta,iphi,4)
      if (ctheta.ge.cphi) then
         x2=theta(itheta)
         y2=phi(iphi-1)
         f2=Tcorner(ir,itheta,iphi,3)
         call lin_interp2d(x1,y1,x2,y2,x3,y3,xtheta,xphi,f1,f2,f3,Tp1)
      else
         x2=theta(itheta-1)
         y2=phi(iphi)
         f2=Tcorner(ir,itheta,iphi,1)
         call lin_interp2d(x1,y1,x2,y2,x3,y3,xtheta,xphi,f1,f2,f3,Tp1)
      endif

      f1=Tcorner(ir,itheta,iphi,6)
      f3=Tcorner(ir,itheta,iphi,8)
      if (ctheta.ge.cphi) then
         f2=Tcorner(ir,itheta,iphi,7)
         call lin_interp2d(x1,y1,x2,y2,x3,y3,xtheta,xphi,f1,f2,f3,Tp2)
      else
         f2=Tcorner(ir,itheta,iphi,5)
         call lin_interp2d(x1,y1,x2,y2,x3,y3,xtheta,xphi,f1,f2,f3,Tp2)
      endif

      call lin_interp1d(r(ir-1),r(ir),xr,Tp1,Tp2,T)

      return
      end
