c     Copyright (C) 2008-2009 Vincent Eymet
c
c     Planet_EMC is free software; you can redistribute it and/or modify
c     it under the terms of the GNU General Public License as published by
c     the Free Software Foundation; either version 3, or (at your option)
c     any later version.

c     Planet_EMC is distributed in the hope that it will be useful,
c     but WITHOUT ANY WARRANTY; without even the implied warranty of
c     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c     GNU General Public License for more details.

c     You should have received a copy of the GNU General Public License
c     along with Planet_EMC; if not, see <http://www.gnu.org/licenses/>
c
      subroutine solar_photon(db,dim,event,toa_ad,
     &     Nr,Ntheta,Nphi,
     &     Nb_gas,Nq,Nb_clouds,
     &     r,theta,phi,rspace,theta0,phi0,Nclouds,Na,
     &     clouds_coord,lambda_gas_lo,lambda_gas_hi,w_quad,
     &     band_min,band_max,spectral_pdf,spectral_cumul,
     &     Tsun,Rsun,dist,
     &     lambda_clouds_lo,lambda_clouds_hi,
     &     clouds_ka,clouds_ks,
     &     clouds_phase,clouds_g,
     &     Nzones,emissivity_zone,
     &     T,r_mc,ref,vri,pf,
     &     r_nodes,r_south_pole,r_north_pole,
     &     coeff_a,coeff_b,coeff_c,
     &     Ncontrib_flux_up,
     &     Ncontrib_flux_dn,
     &     Ncontrib_flux_net,
     &     Ncontrib_flux_toa,
     &     Ncontrib_budget,
     &     flux_idx_up,
     &     flux_idx_dn,
     &     flux_idx_net,
     &     flux_idx_toa,
     &     budget_idx,
     &     flux_contrib_up,
     &     flux_contrib_dn,
     &     flux_contrib_net,
     &     flux_contrib_toa,
     &     budget_contrib,
     &     flux_weight_up,
     &     flux_weight_dn,
     &     flux_weight_net,
     &     flux_weight_toa,
     &     budget_weight,
     &     valid)
      implicit none
      include 'param.inc'
      include 'max.inc'
      include 'mpif.h'
c
c     Purpose: to follow a solar "photon" (energy particle) emitted by the star,
c     from its entry position in the atmosphere, until its gets fully absorbed.
c     Each time the photon crosses a spherical interface, it contributes to the
c     solar flux at that interface; each time it enters a cell, it contributes to
c     the solar radiative budget of this cell.
c
c     Inputs:
c       + dim: dimension of the vectors
c       + event: index of the event
c       + toa_ad: 1 if TOA fluxes computation is required; 0 otherwise
c       + Nr,Ntheta,Nphi: number of cells
c       + Nb_gas: number of spectral narrowbands for gas data
c       + Nq: quadrature order
c       + Nb_clouds: number of spectral narrowbands for clouds data
c       + r,theta,phi: limits of volumic cells
c       + rspace: radius at space level
c       + theta0: latitude of the point where the star is at the zenith (rad)
c       + phi0: longitude of the point where the star is at the zenith (rad)
c       + Nclouds: total number of clouds in the planetary atmosphere
c       + Na: number of angles the phase function has been discretized into
c       + clouds_coord: (r,theta,phi) limits of each cloud
c       + lambda_gas_lo: lower limits of each narrowband spectral interval for gas data
c       + lambda_gas_hi: higher limits of each narrowband spectral interval for gas data
c       + w_quad: quadrature weights
c       + band_min: index of the first narrowband interval that must be taken into account
c       + band_max:  index of the last narrowband interval that must be taken into account
c       + spectral_pdf: discrete pdf function associated to spectral narrowbands
c       + spectral_cumul: discrete cumulative function associated to spectral_pdf
c       + Rsun: radius of the star
c       + Tsun: star emision temperature
c       + dist: distance from star to planet
c       + lambda_clouds_lo: lower limits of each narrowband spectral interval for clouds data
c       + lambda_clouds_hi: higher limits of each narrowband spectral interval for clouds data
c       + clouds_ka: absorption coefficient for every cloud
c       + clouds_ks: scattering coefficient for every cloud
c       + clouds_phase: phase functions
c       + clouds_g: asymetry parameters
c       + Nzones: number of special emissivity zones
c       + emissivity_zone(izone,j): spatial dimensions of special emissivity zones
c         - (izone,1): minimum latitude (rad) of the zone
c         - (izone,2): maximum latitude (rad) of the zone
c         - (izone,3): minimum longitude (rad) of the zone
c         - (izone,4): maximum longitude (rad) of the zone
c       + T: temperatures at cells mass centers
c       + r_mc: radiuses of mass centers
c       + ref: option for reflexion at ground level
c       + vri: option for refraction effects
c       + pf: option for phase function choice
c       + r_nodes: radius (altitude) at each one of the 4 nodes for
c                  each (theta,phi) angular sector
c       + r_south_pole: radius at south pole
c       + r_north_pole: radius at north pole
c       + coeff_a, coeff_b, coeff_c: a, b and c coefficients for each quarter
c
c     Outputs:
c       + Ncontrib_flux_up: number of upward flux contributions for the particle
c       + Ncontrib_flux_dn: number of downward flux contributions for the particle
c       + Ncontrib_flux_net: number of net flux contributions for the particle
c       + Ncontrib_flux_toa: number of TOA flux contributions for the particle
c       + Ncontrib_budget: number of budget contributions for the particle
c       + flux_idx_up: array of spherical interfaces indexes for upward flux
c       + flux_idx_dn: array of spherical interfaces indexes for downward flux
c       + flux_idx_net: array of spherical interfaces indexes for net flux
c       + flux_idx_toa: array of spherical interfaces indexes for TOA flux
c       + budget_idx: array of cells indexes
c       + flux_contrib_up: array of upward flux contributions (energy)
c       + flux_contrib_dn: array of downward flux contributions (energy)
c       + flux_contrib_net: array of net flux contributions (energy)
c       + flux_contrib_toa: array of TOA flux contributions (energy)
c       + budget_contrib: array of budgets contributions (energy)
c       + flux_weight_up: array of upward flux statistical weights
c       + flux_weight_dn: array of downward flux statistical weights
c       + flux_weight_net: array of net flux statistical weights
c       + flux_weight_toa: array of TOA flux statistical weights
c       + budget_weight: array of budgets statistical weights
c       + valid: 1 if generated photon is valid; 0 otherwise
c

c     inputs
      integer db,dim,event,toa_ad
      integer Nr(1:Ntheta_mx,1:Nphi_mx),Ntheta,Nphi,Nb_gas,Nq,Nb_clouds
      integer Nclouds,Na,Nzones
      integer ref,vri,pf
      double precision r(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision theta(0:Ntheta_mx)
      double precision phi(0:Nphi_mx)
      double precision rspace
      double precision theta0,phi0
      double precision clouds_coord(1:Nclouds_mx,1:6)
      double precision lambda_gas_lo(1:Nb_gas_mx)
      double precision lambda_gas_hi(1:Nb_gas_mx)
      double precision w_quad(1:Nq_mx)
      integer band_min,band_max
      double precision spectral_pdf(1:Nb_gas_mx,1:Nq_mx)
      double precision spectral_cumul(0:Ncumul_mx)
      double precision Tsun,Rsun,dist
      double precision lambda_clouds_lo(1:Nb_clouds_mx)
      double precision lambda_clouds_hi(1:Nb_clouds_mx)
      double precision clouds_ka(1:Nclouds_mx,1:Nb_clouds_mx)
      double precision clouds_ks(1:Nclouds_mx,1:Nb_clouds_mx)
      double precision clouds_phase(1:Nclouds_mx,1:Nb_clouds_mx,
     &     1:Nangle_mx,1:2)
      double precision clouds_g(1:Nclouds_mx,1:Nb_clouds_mx)
      double precision emissivity_zone(1:Nzones_mx,1:4)
      double precision T(0:Nz_mx+1,1:Ntheta_mx,1:Nphi_mx)
      double precision r_mc(1:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision r_nodes(1:Ntheta_mx,1:Nphi_mx,1:4)
      double precision r_south_pole,r_north_pole
      double precision coeff_a(1:Ntheta_mx,1:Nphi_mx,1:4)
      double precision coeff_b(1:Ntheta_mx,1:Nphi_mx,1:4)
      double precision coeff_c(1:Ntheta_mx,1:Nphi_mx,1:4)
c     outputs
      integer Ncontrib_flux_up,Ncontrib_flux_dn
      integer Ncontrib_flux_net,Ncontrib_flux_toa
      integer Ncontrib_budget
      integer flux_idx_up(1:Ncontrib_mx,1:5)
      integer flux_idx_dn(1:Ncontrib_mx,1:5)
      integer flux_idx_net(1:Ncontrib_mx,1:5)
      integer flux_idx_toa(1:Ncontrib_mx,1:6)
      integer budget_idx(1:Ncontrib_mx,1:5)
      double precision flux_contrib_up(1:Ncontrib_mx)
      double precision flux_contrib_dn(1:Ncontrib_mx)
      double precision flux_contrib_net(1:Ncontrib_mx)
      double precision flux_contrib_toa(1:Ncontrib_mx)
      double precision budget_contrib(1:Ncontrib_mx)
      double precision flux_weight_up(1:Ncontrib_mx)
      double precision flux_weight_dn(1:Ncontrib_mx)
      double precision flux_weight_net(1:Ncontrib_mx)
      double precision flux_weight_toa(1:Ncontrib_mx)
      double precision budget_weight(1:Ncontrib_mx)
      integer valid
c     MPICH
      integer ierr,errorcode,np,pindex
      integer stat(MPI_STATUS_SIZE)
c     MPICH
c     temp
      integer band,quad
      double precision spectral_weight
      double precision gas_ka(1:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision gas_ks(1:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision n_ref(1:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision ground_emissivity(1:Ntheta_mx,1:Nphi_mx)
      double precision space_emissivity(1:Ntheta_mx,1:Nphi_mx)
      double precision emissivity(1:Nzones_mx)
      double precision solar_power ! W/m/m

      double precision ka_clouds(1:Nclouds_mx)
      double precision ks_clouds(1:Nclouds_mx)
      double precision phase_clouds(1:Nclouds_mx,1:Nangle_mx,1:2)
      double precision g_clouds(1:Nclouds_mx)

      integer strlen,i,app,ioe,edge,ons,sf
      integer method,cont,Nc
      integer ir,itheta,iphi,interface
      integer old_ir,old_itheta,old_iphi
      integer new_ir,new_itheta,new_iphi
      integer ct_old,surface_old
      integer ct,surface,sint,sintt
      integer ct_new,surface_new
      integer cindex(1:Nclouds_mx)
      double precision ccoord(1:Nclouds_mx,1:6)
      double precision ka_cloud(1:Nclouds_mx)
      double precision ks_cloud(1:Nclouds_mx)
      double precision phase(1:Nclouds_mx,1:Nangle_mx,1:2)
      double precision g(1:Nclouds_mx)
      double precision P_in(1:Nmat_mx)
      double precision u_in(1:Nmat_mx)
      double precision P_out(1:Nmat_mx)
      double precision u_out(1:Nmat_mx)
      double precision P_tmp(1:Nmat_mx)
      double precision u1(1:Nmat_mx)
      double precision u2(1:Nmat_mx)
      double precision n_int(1:Nmat_mx)
      double precision n_intt(1:Nmat_mx)
      integer surface_out
      double precision Texch,path_length
      double precision emission_weight
      double precision energy,weight
      double precision energy_in,weight_in
      double precision energy_out,weight_out
      double precision cell_g
      double precision cell_phase(1:Nangle_mx,1:2)
      double precision epsilon_flux
      double precision local_emissivity
      double precision Pos_out(1:Nmat_mx)
      double precision Posc_in(1:Nmat_mx)
      double precision Posc_out(1:Nmat_mx)
      double precision u0(1:Nmat_mx)
      double precision segment(1:Nmat_mx)
      integer col,s_out
      double precision ca(1:4)
      double precision cb(1:4)
      double precision cc(1:4)
      integer v,ir_out,itheta_out,iphi_out,niter,Niter_max
      integer space_out,u_itheta,u_iphi
      character*100 label
c     Debug
      integer j,ngr
      double precision n(1:Nmat_mx),prod,prodt
      double precision sigmaT4,planck_int_wavlng
      double precision n_spher(1:Nmat_mx)
      double precision n_ground(1:Nmat_mx)
c     Debug
      
c     MPICH
c     Get 'np', the number of processes
      call MPI_COMM_SIZE(MPI_COMM_WORLD,np,ierr)
c     Get 'pindex', the index of the current process
      call MPI_COMM_RANK(MPI_COMM_WORLD,pindex,ierr)
c     MPICH

      label='subroutine solar_photon'
      Niter_max=1
c     Debug
      epsilon_flux=1.0D-20 ! W/m/m
c     Debug
c     Debug
      if (db.eq.1) then
         write(*,*) label(1:strlen(label)),' :in'
      endif
c     Debug

      Ncontrib_flux_up=0
      Ncontrib_flux_dn=0
      Ncontrib_flux_net=0
      Ncontrib_flux_toa=0
      Ncontrib_budget=0
      ngr=0
      space_out=0
c     --------------------------------
c     emission of a solar photon
c     --------------------------------

c     choice of narrowband index 'band' and quadrature index 'quad'
      call narrowband_choice(Nb_gas,Nq,band_min,band_max,
     &     spectral_pdf,spectral_cumul,
     &     band,quad,spectral_weight)
c     requiring spectral data for (band,quad) to process 0
c     Debug
c            write(*,*) 'process ',pindex,' requires spectral data'
c     Debug
      call MPI_SEND(pindex,1,MPI_INTEGER,0,1,MPI_COMM_WORLD,ierr)
      call MPI_SEND(0,1,MPI_INTEGER,0,1,MPI_COMM_WORLD,ierr)
      call MPI_SEND(band,1,MPI_INTEGER,0,1,MPI_COMM_WORLD,ierr)
      call MPI_SEND(quad,1,MPI_INTEGER,0,1,MPI_COMM_WORLD,ierr)
      call receive_spectral_data(gas_ka,gas_ks,n_ref,
     &     ground_emissivity,space_emissivity,
     &     emissivity)
      call extract_spectral_cloud_data(Nclouds,Na,
     &     clouds_ka,clouds_ks,clouds_phase,clouds_g,band,
     &     Nb_gas,lambda_gas_lo,lambda_gas_hi,
     &     Nb_clouds,lambda_clouds_lo,lambda_clouds_hi,
     &     ka_clouds,ks_clouds,phase_clouds,g_clouds)
c     energy of the photon
      call solar_emission(band,lambda_gas_lo,lambda_gas_hi,
     &     Tsun,Rsun,dist,solar_power)
c     solar_power: power (W/m/m) emitted by the star that reachs the top
c     of the atmosphere, unit is Watts per square meter of normal surface,
c     per micrometer (at the local wavelength)

c     geometry: entry position
      method=1 !!!! DO NOT TRY to change this value !!!! EVER !!!!! 
      call entry_position_from_space(dim,method,theta0,phi0,
     &     Nr,Ntheta,Nphi,r,theta,phi,rspace,itheta,iphi,P_in,u_in,
     &     emission_weight)

c     energy: radiative power carried by the particle, in W/m/m
c     m: square meters of planetary cross-section
      energy=solar_power*space_emissivity(itheta,iphi) ! W/m/m
c     weight: statistical weight associated to sampling method (unitless)
      weight=emission_weight ! m
     &     *spectral_weight ! unitless = [m]

c     update flux contribution arrays
      call update_solar_flux(dim,toa_ad,
     &     Ntheta,Nr(itheta,iphi),itheta,iphi,2,
     &     r,theta,phi,0,0,0,
     &     r_nodes,r_south_pole,r_north_pole,coeff_a,coeff_b,coeff_c,
     &     energy,weight,
     &     band,quad,P_in,u_in,
     &     Ncontrib_flux_up,
     &     Ncontrib_flux_dn,
     &     Ncontrib_flux_net,
     &     Ncontrib_flux_toa,
     &     flux_idx_up,
     &     flux_idx_dn,
     &     flux_idx_net,
     &     flux_idx_toa,
     &     flux_contrib_up,
     &     flux_contrib_dn,
     &     flux_contrib_net,
     &     flux_contrib_toa,
     &     flux_weight_up,
     &     flux_weight_dn,
     &     flux_weight_net,
     &     flux_weight_toa)
      ct=1
      surface=2
      ir=Nr(itheta,iphi) ! (itheta,iphi): output from "entry_position_from_space" routine call

c     --------------------------------
c     propagation of a solar photon
c     --------------------------------

      cont=1
      do while (cont.eq.1)

         energy_in=energy
         weight_in=weight
c         if (((iphi.eq.Nphi).or.(iphi.eq.1)).and.
c     &        (dabs(P_in(3)-2.0D+0*pi).lt.1.0D-8)) then
c            iphi=1
c            P_in(3)=0.0D+0
c         endif

c     Debug
         if (db.eq.1) then
            write(*,*) 'P_in=',(P_in(j),j=1,dim)
         endif
c     Debug
         if (ct.eq.2) then
            call extract_coeffs(itheta,iphi,coeff_a,coeff_b,coeff_c,
     &           ca,cb,cc)
            if (ir.eq.1) then
               call need2approximate(dim,Ntheta,r,theta,phi,
     &              r_nodes,r_south_pole,r_north_pole,itheta,iphi,
     &              app)        ! app=1: need to take into account the real surface at ground level
            else
               app=0
            endif
            call is_on_edge(P_in,r(ir-1,itheta,iphi),r(ir,itheta,iphi),
     &           theta(itheta-1),theta(itheta),
     &           phi(iphi-1),phi(iphi),app,ca,cb,cc,ioe,edge)
         else
            ioe=0
         endif

c     Debug
         if ((surface.lt.1).or.(surface.gt.6)) then
            write(*,*) 'Error from ',
     &           label(1:strlen(label)),' :'
            write(*,*) 'surface=',surface
            write(*,*) 'before entering new_cell'
            stop
         endif
c     Debug
         old_ir=ir
         old_itheta=itheta
         old_iphi=iphi
         ct_old=ct
         surface_old=surface
         call new_cell(db,dim,ioe,edge,P_in,u_in,Nr,Ntheta,Nphi,
     &        r,theta,phi,
     &        r_nodes,r_south_pole,r_north_pole,
     &        coeff_a,coeff_b,coeff_c,
     &        ct,surface,ir,itheta,iphi,P_tmp,
     &        ct_new,surface_new,new_ir,new_itheta,new_iphi)
         call copy_vector(dim,P_tmp,P_in)

c     Debug
         if ((surface_new.lt.1).or.(surface_new.gt.6)) then
            write(*,*) 'Error from ',
     &           label(1:strlen(label)),' :'
            write(*,*) 'surface_new=',surface_new
            write(*,*) 'right after new_cell'
            stop
         endif
         if (db.eq.1) then
            write(*,*) 'new cell, ct=',ct,' ct_new=',ct_new
            write(*,*) 'surface=',surface,' surface_new=',surface_new
            write(*,*) 'P_in=',(P_in(j),j=1,dim)
            write(*,*) 'u_in=',(u_in(j),j=1,dim)
         endif
c     Debug
         ir=new_ir
         itheta=new_itheta
         iphi=new_iphi
         ct=ct_new
         surface=surface_new
c     Debug
         if ((surface.lt.1).or.(surface.gt.6)) then
            write(*,*) 'Error from ',
     &           label(1:strlen(label)),' :'
            write(*,*) 'surface=',surface
            write(*,*) 'position 2'
            stop
         endif
c     Debug
         
         if (ct.eq.1) then ! particle reached ground or space
            if ((surface.eq.1).or.(surface.eq.2)) then
               if (surface.eq.1) then
                  ngr=ngr+1
c                  if (ngr.gt.1000) then
c                     write(*,*) 'Error from ',
c     &                    label(1:strlen(label)),' :'
c                     write(*,*) 'particle can not leave ground'
c                     write(*,*) 'P_in=',(P_in(j),j=1,dim)
c                     write(*,*) 'u_in=',(u_in(j),j=1,dim)
c                     write(*,*) 'ct=',ct
c                     write(*,*) 'surface=',surface
c                     write(*,*) 'ir=',ir
c                     write(*,*) 'itheta=',itheta
c                     write(*,*) 'iphi=',iphi
c                     stop
c                  endif

c     Debug
                  if (db.eq.1) then
                     write(*,*) 
                     write(*,*) 'particle reached ground'
                     write(*,*) 
                  endif
c     testing the production of normal directions -> ok !
c                  call need2approximate(dim,Ntheta,r,theta,phi,
c     &                 r_nodes,r_south_pole,r_north_pole,itheta,iphi,app)
c                  call extract_coeffs(itheta,iphi,
c     &                 coeff_a,coeff_b,coeff_c,ca,cb,cc)
c                  if ((surface.eq.1).and.(app.eq.1)) then
c                     call ground_normal(dim,P_in,
c     &                    theta(itheta-1),theta(itheta),
c     &                    phi(iphi-1),phi(iphi),ca,cb,cc,n_ground)
c                     write(*,*) 'n_ground=',(n_ground(j),j=1,dim)
c                  else
c                     call inside_normal(surface,P_in,n_spher)
c                     write(*,*) 'n_spher=',(n_spher(j),j=1,dim)
c                  endif
c     Debug
                  interface=0
               else ! surface=2, particle reached space
c     Debug
                  if (db.eq.1) then
                     write(*,*) 'particle reached space'
                  endif
c     Debug
                  interface=Nr(itheta,iphi)
               endif
c     Debug
               if ((surface.lt.1).or.(surface.gt.6)) then
                  write(*,*) 'Error from ',
     &                 label(1:strlen(label)),' :'
                  write(*,*) 'surface=',surface
                  write(*,*) 'before surface_reflexion'
                  stop
               endif
c     Debug
               call surface_reflexion(dim,ref,Ntheta,Nphi,
     &              r,theta,phi,
     &              r_nodes,r_south_pole,r_north_pole,
     &              coeff_a,coeff_b,coeff_c,
     &              itheta,iphi,surface,P_in,u_in,
     &              ground_emissivity,space_emissivity,
     &              Nzones,emissivity_zone,emissivity,
     &              energy_in,weight_in,
     &              P_out,u_out,energy_out,weight_out)
c     Debug
               if ((surface.lt.1).or.(surface.gt.6)) then
                  write(*,*) 'Error from ',
     &                 label(1:strlen(label)),' :'
                  write(*,*) 'surface=',surface
                  write(*,*) 'after surface_reflexion'
                  stop
               endif
c     Debug
c     Debug
               if (db.eq.1) then
                  write(*,*) 'u_in=',(u_in(j),j=1,dim)
                  write(*,*) 'u_out=',(u_out(j),j=1,dim)
                  write(*,*) 'P_in=',(P_in(j),j=1,dim)
                  write(*,*) 'P_out=',(P_out(j),j=1,dim)
               endif
c     Debug
c     Debug
               if ((surface.lt.1).or.(surface.gt.6)) then
                  write(*,*) 'Error from ',
     &                 label(1:strlen(label)),' :'
                  write(*,*) 'surface=',surface
                  write(*,*) 'before update_solar_flux'
                  stop
               endif
c     Debug
               call update_solar_flux(dim,toa_ad,
     &              Ntheta,interface,
     &              itheta,iphi,surface,
     &              r,theta,phi,0,0,0,
     &              r_nodes,r_south_pole,
     &              r_north_pole,coeff_a,coeff_b,coeff_c,
     &              energy_out,weight_out,band,quad,
     &              P_out,u_out,
     &              Ncontrib_flux_up,
     &              Ncontrib_flux_dn,
     &              Ncontrib_flux_net,
     &              Ncontrib_flux_toa,
     &              flux_idx_up,
     &              flux_idx_dn,
     &              flux_idx_net,
     &              flux_idx_toa,
     &              flux_contrib_up,
     &              flux_contrib_dn,
     &              flux_contrib_net,
     &              flux_contrib_toa,
     &              flux_weight_up,
     &              flux_weight_dn,
     &              flux_weight_net,
     &              flux_weight_toa)
c     Debug
               if ((surface.lt.1).or.(surface.gt.6)) then
                  write(*,*) 'Error from ',
     &                 label(1:strlen(label)),' :'
                  write(*,*) 'surface=',surface
                  write(*,*) 'after update_solar_flux'
                  stop
               endif
c     Debug
            else
               write(*,*) 'Error from ',
     &              label(1:strlen(label)),' :'
               write(*,*) 'cell type=',ct
               write(*,*) 'with surface=',surface
               stop
            endif
         else if (ct.eq.2) then ! particle reached a gas cell
c     Debug
c      if (db.eq.1) then
c         write(*,*) 'particle reached gas cell, ir=',ir
c      endif
c     Debug
c     identify clouds that intersect the current cell
c     Debug
c      if (db.eq.1) then
c         write(*,*) 'ok3',itheta,iphi
c      endif
c     Debug
            niter=1
 222        continue

c     refraction
            if (vri.eq.1) then
               call copy_vector(dim,u_in,u1)
c     refraction must be taken into account when the particle goes from a
c     gas volume to another gas volume or from space to a gas volume
               if (((ct_old.eq.2).and.(ct.eq.2)).or.
     &              ((ct_old.eq.1).and.(surface_old.eq.2)
     &              .and.(ct.eq.2))) then
                  sint=surface
 444              continue
                  call inside_normal(sint,P_in,n_int)
                  call scalar_product(dim,u1,n_int,prod)
                  if (prod.lt.0.0D+0) then
                     sf=0
                     do sintt=1,6
                        call inside_normal(sint,P_in,n_intt)
                        call scalar_product(dim,u1,n_intt,prodt)
                        if (prodt.gt.0.0D+0) then
                           sf=1
                           sint=sintt
                           call copy_vector(dim,n_intt,n_int)
                           write(*,*) 'warning from routine ',
     &                          label(1:strlen(label)),' :'
                           write(*,*) 'sint was modified'
                           goto 444
                        endif
                     enddo ! sintt
                     if (sf.eq.0) then
                        write(*,*) 'Error from ',
     &                       label(1:strlen(label)),' :'
                        write(*,*) '@ refraction direction'
                        write(*,*) 'no suitable surface found'
                        stop
                     endif
                  endif
c     n_int is directed inside cell (ir,itheta,iphi)
                  if ((ct_old.eq.2).and.(ct.eq.2)) then
c     for particles that go from a gas cell to another gas cell
                     call refraction(dim,
     &                    n_ref(old_ir,old_itheta,old_iphi),
     &                    n_ref(ir,itheta,iphi),
     &                    u1,n_int,u2)
                  else if ((ct_old.eq.1).and.(surface_old.eq.2)
     &                    .and.(ct.eq.2)) then
c     for particles that go from space to a gas cell
                     call refraction(dim,
     &                    1.0D+0,
     &                    n_ref(ir,itheta,iphi),
     &                    u1,n_int,u2)
                  endif
                  call copy_vector(dim,u2,u_in)
               endif
            endif

            call identify_clouds(Nclouds,clouds_coord,
     &           r(ir-1,itheta,iphi),r(ir,itheta,iphi),
     &           theta(itheta-1),theta(itheta),phi(iphi-1),phi(iphi),
     &           Nc,cindex,ccoord)
            call get_cloud_data(Nclouds,Na,Nc,cindex,
     &           ka_clouds,ks_clouds,phase_clouds,g_clouds,
     &           ka_cloud,ks_cloud,phase,g)
c     Debug
c      if (db.eq.1) then
c         write(*,*) 'ok4',itheta,iphi
c      endif
c     Debug 'ok1'
c     Debug
c     pick a scattering path within the current cell
c     Debug
            if ((surface.lt.1).or.(surface.gt.6)) then
               write(*,*) 'Error from ',
     &              label(1:strlen(label)),' :'
               write(*,*) 'surface=',surface
               write(*,*) 'before volume_exchange_position'
               stop
            endif
c     Debug
            call volume_exchange_position(db,dim,event,Nr,Ntheta,Nphi,
     &           r,theta,phi,ir,itheta,iphi,T,r_mc,P_in,u_in,surface,
     &           pf,Nc,Na,phase,g,ccoord,
     &           gas_ka(ir,itheta,iphi),
     &           gas_ks(ir,itheta,iphi),
     &           ka_cloud,ks_cloud,
     &           r_nodes,r_south_pole,r_north_pole,
     &           coeff_a,coeff_b,coeff_c,
     &           energy_in,weight_in,
     &           P_out,u_out,surface_out,Texch,path_length,
     &           energy_out,weight_out,valid)
c     Debug
            if ((valid.ne.0).and.
     &           ((surface_out.lt.1).or.(surface_out.gt.6))) then
               write(*,*) 'Error from ',
     &              label(1:strlen(label)),' :'
               write(*,*) 'surface_out=',surface_out
               write(*,*) 'after volume_exchange_position'
               write(*,*) 'valid=',valid
               stop
            endif
c     Debug
            if ((valid.eq.0).and.(niter.le.Niter_max)) then
               niter=niter+1
               call last_chance(dim,event,Nr,Ntheta,Nphi,r,theta,phi,
     &              rspace,r_nodes,r_south_pole,r_north_pole,
     &              coeff_a,coeff_b,coeff_c,P_in,u_in,
     &              ir_out,itheta_out,iphi_out,P_out,u_out,s_out,v)
c     Debug
               if ((s_out.lt.1).or.(s_out.gt.6)) then
                  write(*,*) 'Error from ',
     &                 label(1:strlen(label)),' :'
                  write(*,*) 's_out=',s_out
                  write(*,*) 'after last_chance'
                  stop
               endif
c     Debug
               if (v.eq.1) then
                  ir=ir_out
                  itheta=itheta_out
                  iphi=iphi_out
                  call copy_vector(dim,P_out,P_in)
                  call copy_vector(dim,u_out,u_in)
                  surface=s_out
                  goto 222
               else
                  valid=0
               endif ! v=1
            endif

            if (valid.eq.0) then
               goto 666
            endif
c     Debug
            if (db.eq.1) then
               write(*,*) 'surface_in=',surface,
     &              ' surface_out=',surface_out
            endif
c     Debug
c     Debug
            if ((surface_out.lt.1).or.(surface_out.gt.6)) then
               write(*,*) 'Error from ',
     &              label(1:strlen(label)),' :'
               write(*,*) 'surface_out=',surface_out
               write(*,*) 'position 3'
               stop
            endif
c     Debug
c     update solar fluxes for spherical interfaces
            if ((surface_out.eq.1).or.(surface_out.eq.2)) then
               if (surface_out.eq.1) then
                  interface=ir-1
               else
                  interface=ir
               endif

               if ((surface_out.eq.2).and.
     &              (ir.eq.Nr(itheta,iphi))) then ! if particle reached space level
                  space_out=1
c     identify the angular sector direction "u_out" belongs to
                  call sort_direction(dim,Ntheta,Nphi,theta,phi,u_out,
     &                 u_itheta,u_iphi)
c     (u_itheta,u_iphi) are the indexes of the angular sector "u_out" belongs to
               else ! all other cases
                  space_out=0
                  u_itheta=0
                  u_iphi=0
               endif
c     Debug
c      if (db.eq.1) then
c         write(*,*) 'update solar flux, interface=',interface
c      endif
c     Debug
c     Debug
               if ((surface_out.lt.1).or.(surface_out.gt.6)) then
                  write(*,*) 'Error from ',
     &                 label(1:strlen(label)),' :'
                  write(*,*) 'surface_out=',surface_out
                  write(*,*) 'position 4'
                  stop
               endif
c     Debug
               call update_solar_flux(dim,toa_ad,
     &              Ntheta,interface,
     &              itheta,iphi,surface_out,
     &              r,theta,phi,space_out,u_itheta,u_iphi,
     &              r_nodes,r_south_pole,
     &              r_north_pole,coeff_a,coeff_b,coeff_c,
     &              energy_out,weight_out,band,quad,
     &              P_out,u_out,
     &              Ncontrib_flux_up,
     &              Ncontrib_flux_dn,
     &              Ncontrib_flux_net,
     &              Ncontrib_flux_toa,
     &              flux_idx_up,
     &              flux_idx_dn,
     &              flux_idx_net,
     &              flux_idx_toa,
     &              flux_contrib_up,
     &              flux_contrib_dn,
     &              flux_contrib_net,
     &              flux_contrib_toa,
     &              flux_weight_up,
     &              flux_weight_dn,
     &              flux_weight_net,
     &              flux_weight_toa)
c     Debug
c               if (space_out.eq.1) then
c                  write(*,*) 
c                  write(*,*) flux_idx_toa(Ncontrib_flux_toa,3)
c                  write(*,*) flux_idx_toa(Ncontrib_flux_toa,4)
c                  write(*,*) flux_contrib_toa(Ncontrib_flux_toa)
c                  write(*,*) flux_weight_toa(Ncontrib_flux_toa)
c                  write(*,*) 'energy_out=',energy_out
c               endif
c     Debug

c     Debug
               if ((surface_out.lt.1).or.(surface_out.gt.6)) then
                  write(*,*) 'Error from ',
     &                 label(1:strlen(label)),' :'
                  write(*,*) 'surface_out=',surface_out
                  write(*,*) 'position 5'
                  stop
               endif
c     Debug
            endif
c     update solar budgets
            call update_solar_budget(dim,ir,itheta,iphi,
     &           energy_in,weight_in,energy_out,weight_out,band,quad,
     &           Ncontrib_budget,budget_idx,budget_contrib,
     &           budget_weight)
         else
            write(*,*) 'Error from ',
     &           label(1:strlen(label)),' :'
            write(*,*) 'cell type=',ct
            stop
         endif ! ct=1 or 2
         
         call copy_vector(dim,P_out,P_in)
         call copy_vector(dim,u_out,u_in)
         surface=surface_out
         energy=energy_out
         weight=weight_out

         if (energy.lt.epsilon_flux) then
            cont=0
         endif
c     Debug
         if ((surface.lt.1).or.(surface.gt.6)) then
            write(*,*) 'Error from ',
     &           label(1:strlen(label)),' :'
            write(*,*) 'surface=',surface
            write(*,*) 'position 6'
            stop
         endif
c     Debug

      enddo !  while cont=1
c     Debug
      if (db.eq.1) then
         write(*,*) label(1:strlen(label)),' :out'
      endif
c     Debug

 666  continue
      return
      end



      subroutine update_solar_budget(dim,ir,itheta,iphi,
     &     energy_in,weight_in,energy_out,weight_out,band,quad,
     &     Ncontrib_budget,budget_idx,budget_contrib,budget_weight)
      implicit none
      include 'max.inc'
c
c     Purpose: to update solar budget contribution arrays
c
c     Inputs:
c       + dim: dimension of the vectors
c       + ir,itheta,iphi: index of the spherical cell the budget has to be updated for
c       + energy_in: energy carried by the particle before interaction ! W/m/m
c       + weight_in: statistical weight of the particle before interaction
c       + energy_out: energy carried by the particle after interaction ! W/m/m
c       + weight_out: statistical weight of the particle after interaction
c       + band: gas data narowband index the solar photon has been emitted into
c       + quad: quadrature index
c     I/O:
c       + Ncontrib_budget: number of budget contributions for the particle
c       + budget_idx: array of spherical interfaces indexes for budget
c       + budget_contrib: array of budget contributions (energy) ! W/m/m
c       + budget_weight: array of budget statistical weights
c     
c     I/O
      integer dim,ir,itheta,iphi,band,quad
      double precision energy_in,weight_in,energy_out,weight_out
      integer Ncontrib_budget
      integer budget_idx(1:Ncontrib_mx,1:5)
      double precision budget_contrib(1:Ncontrib_mx)
      double precision budget_weight(1:Ncontrib_mx)
c     temp
      integer strlen
      character*100 label
      label='subroutine update_solar_budget'

      Ncontrib_budget=Ncontrib_budget+1
      if (Ncontrib_budget.gt.Ncontrib_mx) then
         write(*,*) 'Error from ',label(1:strlen(label)),' :'
         write(*,*) 'Ncontrib_budget=',Ncontrib_budget
         write(*,*) ' > Ncontrib_mx'
         stop
      endif
      budget_idx(Ncontrib_budget,1)=ir
      budget_idx(Ncontrib_budget,2)=itheta
      budget_idx(Ncontrib_budget,3)=iphi
      budget_idx(Ncontrib_budget,4)=band
      budget_idx(Ncontrib_budget,5)=quad
      budget_contrib(Ncontrib_budget)=energy_in-energy_out ! W/m/m
      if (budget_contrib(Ncontrib_budget).lt.0.0D+0) then
         write(*,*) 'Error from ',label(1:strlen(label)),' :'
         write(*,*) 'budget_contrib(',Ncontrib_budget,')-',
     &        budget_contrib(Ncontrib_budget)
         stop
      endif
      budget_weight(Ncontrib_budget)=weight_in ! m

      return
      end


      
      subroutine update_solar_flux(dim,toa_ad,
     &     Ntheta,ir,itheta,iphi,stype,
     &     r,theta,phi,space_out,u_itheta,u_iphi,
     &     r_nodes,r_south_pole,r_north_pole,coeff_a,coeff_b,coeff_c,
     &     energy,weight,band,quad,P,u,
     &     Ncontrib_flux_up,
     &     Ncontrib_flux_dn,
     &     Ncontrib_flux_net,
     &     Ncontrib_flux_toa,
     &     flux_idx_up,
     &     flux_idx_dn,
     &     flux_idx_net,
     &     flux_idx_toa,
     &     flux_contrib_up,
     &     flux_contrib_dn,
     &     flux_contrib_net,
     &     flux_contrib_toa,
     &     flux_weight_up,
     &     flux_weight_dn,
     &     flux_weight_net,
     &     flux_weight_toa)
      implicit none
      include 'max.inc'
c
c     Purpose: to update solar flux contribution arrays
c
c     Inputs:
c       + dim: dimension of the vectors
c       + toa_ad: 1 if TOA fluxes computation is required; 0 otherwise
c       + Ntheta
c       + ir,itheta,iphi: index of the spherical interface the flux needs
c         to be updated for
c       + stype: 1 if interface is the ground, 2 is interface is space
c       + r,theta,phi: cells discretization
c       + space_out: 1 if the particle exits at space level, 0 otherwise
c       + (u_itheta,u_iphi): indexes of the angular sector the propagation
c                            direction belongs to, in the case space_out=1
c       + r_nodes: radius (altitude) at each one of the 4 nodes for
c                  each (theta,phi) angular sector
c       + r_south_pole: radius at south pole
c       + r_north_pole: radius at north pole
c       + coeff_a, coeff_b, coeff_c: a, b and c coefficients for each quarter
c       + energy: energy carried by the particle at the interface ! W/m/m
c       + weight: statistical weight of the particle at the interface
c       + band: gas data narowband index the solar photon has been emitted into
c       + quad: quadrature index
c       + P: position the particle crosses the interface at
c       + u: direction of propagation of the particule at the interface
c     I/O:
c       + Ncontrib_flux_up: number of upward flux contributions for the particle
c       + Ncontrib_flux_dn: number of downward flux contributions for the particle
c       + Ncontrib_flux_net: number of net flux contributions for the particle
c       + Ncontrib_flux_toa: number of TOA flux contributions for the particle
c       + flux_idx_up: array of spherical interfaces indexes for upward flux
c       + flux_idx_dn: array of spherical interfaces indexes for downward flux
c       + flux_idx_net: array of spherical interfaces indexes for net flux
c       + flux_idx_toa: array of indexes for TOA flux
c       + flux_contrib_up: array of upward flux contributions (energy) ! W/m/m
c       + flux_contrib_dn: array of downward flux contributions (energy) ! W/m/m
c       + flux_contrib_net: array of net flux contributions (energy) ! W/m/m
c       + flux_contrib_toa: array of TOA flux contributions (energy) ! W/m/m
c       + flux_weight_up: array of upward flux statistical weights
c       + flux_weight_dn: array of downward flux statistical weights
c       + flux_weight_net: array of net flux statistical weights
c       + flux_weight_toa: array of TOA flux statistical weights
c

c     I/O
      integer dim,toa_ad,ir,itheta,iphi,band,quad,Ntheta,stype
      double precision r(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision theta(0:Ntheta_mx)
      double precision phi(0:Nphi_mx)
      integer space_out,u_itheta,u_iphi
      double precision r_nodes(1:Ntheta_mx,1:Nphi_mx,1:4)
      double precision r_south_pole,r_north_pole
      double precision coeff_a(1:Ntheta_mx,1:Nphi_mx,1:4)
      double precision coeff_b(1:Ntheta_mx,1:Nphi_mx,1:4)
      double precision coeff_c(1:Ntheta_mx,1:Nphi_mx,1:4)
      double precision energy,weight
      double precision P(1:Nmat_mx)
      double precision u(1:Nmat_mx)
      integer Ncontrib_flux_up
      integer Ncontrib_flux_dn
      integer Ncontrib_flux_net
      integer Ncontrib_flux_toa
      integer flux_idx_up(1:Ncontrib_mx,1:5)
      integer flux_idx_dn(1:Ncontrib_mx,1:5)
      integer flux_idx_net(1:Ncontrib_mx,1:5)
      integer flux_idx_toa(1:Ncontrib_mx,1:6)
      double precision flux_contrib_up(1:Ncontrib_mx)
      double precision flux_contrib_dn(1:Ncontrib_mx)
      double precision flux_contrib_net(1:Ncontrib_mx)
      double precision flux_contrib_toa(1:Ncontrib_mx)
      double precision flux_weight_up(1:Ncontrib_mx)
      double precision flux_weight_dn(1:Ncontrib_mx)
      double precision flux_weight_net(1:Ncontrib_mx)
      double precision flux_weight_toa(1:Ncontrib_mx)
c     temp
      integer strlen,app,i
      double precision n(1:Nmat_mx)
      double precision sp
      double precision ca(1:4)
      double precision cb(1:4)
      double precision cc(1:4)
      character*100 label
      label='subroutine update_solar_flux'

c     Debug
c      write(*,*) label(1:strlen(label)),' with ir=',ir
c     Debug
      call need2approximate(dim,Ntheta,r,theta,phi,
     &     r_nodes,r_south_pole,r_north_pole,itheta,iphi,app)
      if ((app.eq.1).and.(stype.eq.1)) then
         call extract_coeffs(itheta,iphi,coeff_a,coeff_b,coeff_c,
     &        ca,cb,cc)
         call ground_normal(dim,P,theta(itheta-1),theta(itheta),
     &        phi(iphi-1),phi(iphi),ca,cb,cc,n)
      else
         call inside_normal(2,P,n)
      endif
      call scalar_product(dim,u,n,sp)

      if (sp.gt.0.0D+0) then ! downward flux
         Ncontrib_flux_dn=Ncontrib_flux_dn+1
         if (Ncontrib_flux_dn.gt.Ncontrib_mx) then
            write(*,*) 'Error from ',label(1:strlen(label)),' :'
            write(*,*) 'Ncontrib_flux_dn=',Ncontrib_flux_dn
            write(*,*) ' > Ncontrib_mx'
            write(*,*) 'P=',(P(i),i=1,dim)
            write(*,*) 'u=',(u(i),i=1,dim)
            stop
         endif
         flux_idx_dn(Ncontrib_flux_dn,1)=ir
         flux_idx_dn(Ncontrib_flux_dn,2)=itheta
         flux_idx_dn(Ncontrib_flux_dn,3)=iphi
         flux_idx_dn(Ncontrib_flux_dn,4)=band
         flux_idx_dn(Ncontrib_flux_dn,5)=quad
         flux_contrib_dn(Ncontrib_flux_dn)=energy ! W/m/m
         flux_weight_dn(Ncontrib_flux_dn)=weight ! m
      else ! upward flux
         Ncontrib_flux_up=Ncontrib_flux_up+1
         if (Ncontrib_flux_up.gt.Ncontrib_mx) then
            write(*,*) 'Error from ',label(1:strlen(label)),' :'
            write(*,*) 'Ncontrib_flux_up=',Ncontrib_flux_up
            write(*,*) ' > Ncontrib_mx'
            write(*,*) 'P=',(P(i),i=1,dim)
            write(*,*) 'u=',(u(i),i=1,dim)
            stop
         endif
         flux_idx_up(Ncontrib_flux_up,1)=ir
         flux_idx_up(Ncontrib_flux_up,2)=itheta
         flux_idx_up(Ncontrib_flux_up,3)=iphi
         flux_idx_up(Ncontrib_flux_up,4)=band
         flux_idx_up(Ncontrib_flux_up,5)=quad
         flux_contrib_up(Ncontrib_flux_up)=energy ! W/m/m
         flux_weight_up(Ncontrib_flux_up)=weight ! m
      endif
c     In any case, compute contribution to the net flux
      Ncontrib_flux_net=Ncontrib_flux_net+1
      if (Ncontrib_flux_net.gt.Ncontrib_mx) then
         write(*,*) 'Error from ',label(1:strlen(label)),' :'
         write(*,*) 'Ncontrib_flux_net=',Ncontrib_flux_net
         write(*,*) ' > Ncontrib_mx'
         write(*,*) 'P=',(P(i),i=1,dim)
         write(*,*) 'u=',(u(i),i=1,dim)
         stop
      endif
      flux_idx_net(Ncontrib_flux_net,1)=ir
      flux_idx_net(Ncontrib_flux_net,2)=itheta
      flux_idx_net(Ncontrib_flux_net,3)=iphi
      flux_idx_net(Ncontrib_flux_net,4)=band
      flux_idx_net(Ncontrib_flux_net,5)=quad
      if (sp.lt.0.0D+0) then
         flux_contrib_net(Ncontrib_flux_net)=energy ! W/m/m
      else
         flux_contrib_net(Ncontrib_flux_net)=-energy ! W/m/m
      endif
      flux_weight_net(Ncontrib_flux_net)=weight ! m
c     Also compute contribution to the flux at the top of the atmosphere
      if ((space_out.eq.1).and.(toa_ad.eq.1)) then
         Ncontrib_flux_toa=Ncontrib_flux_toa+1
         if (Ncontrib_flux_toa.gt.Ncontrib_mx) then
            write(*,*) 'Error from ',label(1:strlen(label)),' :'
            write(*,*) 'Ncontrib_flux_toa=',Ncontrib_flux_toa
            write(*,*) ' > Ncontrib_mx'
            write(*,*) 'P=',(P(i),i=1,dim)
            write(*,*) 'u=',(u(i),i=1,dim)
            stop
         endif
         flux_idx_toa(Ncontrib_flux_toa,1)=itheta
         flux_idx_toa(Ncontrib_flux_toa,2)=iphi
         flux_idx_toa(Ncontrib_flux_toa,3)=u_itheta
         flux_idx_toa(Ncontrib_flux_toa,4)=u_iphi
         flux_idx_toa(Ncontrib_flux_toa,5)=band
         flux_idx_toa(Ncontrib_flux_toa,6)=quad
         flux_contrib_toa(Ncontrib_flux_toa)=energy ! W/m/m
         flux_weight_toa(Ncontrib_flux_toa)=weight ! W/m/m
      endif
      
      return
      end



      subroutine update_solar_results(db,toa_ad,
     &     Ncontrib_flux_up,
     &     Ncontrib_flux_dn,
     &     Ncontrib_flux_net,
     &     Ncontrib_flux_toa,
     &     Ncontrib_budget,
     &     flux_idx_up,
     &     flux_idx_dn,
     &     flux_idx_net,
     &     flux_idx_toa,
     &     budget_idx,
     &     flux_contrib_up,
     &     flux_contrib_dn,
     &     flux_contrib_net,
     &     flux_contrib_toa,
     &     budget_contrib,
     &     flux_weight_up,
     &     flux_weight_dn,
     &     flux_weight_net,
     &     flux_weight_toa,
     &     budget_weight,
     &     lambda_gas_lo,
     &     lambda_gas_hi,
     &     w_quad,
     &     sum_flux_up,
     &     sum_flux_dn,
     &     sum_flux_net,
     &     sum_flux_toa,
     &     sum_budget,
     &     sum2_flux_up,
     &     sum2_flux_dn,
     &     sum2_flux_net,
     &     sum2_flux_toa,
     &     sum2_budget)
      implicit none
      include 'max.inc'
c
c     Purpose: to update global results (budget and fluxes) due to solar emission
c     when a solar photon has been totally absorbed
c
c     Inputs:
c       + toa_ad: 1 if TOA fluxes computation is required; 0 otherwise
c       + Ncontrib_flux_up: number of upward flux contributions for the particle
c       + Ncontrib_flux_dn: number of downward flux contributions for the particle
c       + Ncontrib_flux_net: number of net flux contributions for the particle
c       + Ncontrib_flux_toa: number of TOA flux contributions for the particle
c       + Ncontrib_budget: number of budget contributions for the particle
c       + flux_idx_up: array of spherical interfaces indexes for upward flux
c       + flux_idx_dn: array of spherical interfaces indexes for downward flux
c       + flux_idx_net: array of spherical interfaces indexes for net flux
c       + flux_idx_toa: array of spherical interfaces indexes for TOA flux
c       + budget_idx: array of cells indexes
c       + flux_contrib_up: array of upward flux contributions (energy) ! W/m/m
c       + flux_contrib_dn: array of downward flux contributions (energy) ! W/m/m
c       + flux_contrib_net: array of net flux contributions (energy) ! W/m/m
c       + flux_contrib_toa: array of TOA flux contributions (energy) ! W/m/m
c       + budget_contrib: array of budgets contributions (energy) ! W/m/m
c       + flux_weight_up: array of upward flux statistical weights
c       + flux_weight_dn: array of downward flux statistical weights
c       + flux_weight_net: array of net flux statistical weights
c       + flux_weight_toa: array of TOA flux statistical weights
c       + budget_weight: array of budgets statistical weights
c       + lambda_gas_lo: lower limits of each narrowband spectral interval for gas data
c       + lambda_gas_hi: higher limits of each narrowband spectral interval for gas data
c       + w_quad: quadrature weights
c
c     Outputs:
c       + sum_flux_up: sum of contribution for solar upward flux at each spherical interface ! W/m
c       + sum_flux_dn: sum of contribution for solar downard flux at each spherical interface ! W/m
c       + sum_flux_net: sum of contribution for solar net flux at each spherical interface ! W/m
c       + sum_flux_toa: sum of contribution for solar TOA flux in each column ! W/m
c       + sum_budget: sum of contribution for solar budget in each cell ! W/m
c       + sum2_flux_up: sum of squared contribution for solar upward flux at each spherical interface ! (W/m)
c       + sum2_flux_dn: sum of squared contribution for solar downard flux at each spherical interface ! (W/m)
c       + sum2_flux_net: sum of squared contribution for solar net flux at each spherical interface ! (W/m)
c       + sum2_flux_toa: sum of squared contribution for solar TOA flux in each column ! (W/m)
c       + sum2_budget: sum of squared contribution for solar budget in each cell ! (W/m)
c

c     Inputs
      integer db,toa_ad
      integer Ncontrib_flux_up,Ncontrib_flux_dn
      integer Ncontrib_flux_net,Ncontrib_flux_toa
      integer Ncontrib_budget
      integer flux_idx_up(1:Ncontrib_mx,1:5)
      integer flux_idx_dn(1:Ncontrib_mx,1:5)
      integer flux_idx_net(1:Ncontrib_mx,1:5)
      integer flux_idx_toa(1:Ncontrib_mx,1:6)
      integer budget_idx(1:Ncontrib_mx,1:5)
      double precision flux_contrib_up(1:Ncontrib_mx)
      double precision flux_contrib_dn(1:Ncontrib_mx)
      double precision flux_contrib_net(1:Ncontrib_mx)
      double precision flux_contrib_toa(1:Ncontrib_mx)
      double precision budget_contrib(1:Ncontrib_mx)
      double precision flux_weight_up(1:Ncontrib_mx)
      double precision flux_weight_dn(1:Ncontrib_mx)
      double precision flux_weight_net(1:Ncontrib_mx)
      double precision flux_weight_toa(1:Ncontrib_mx)
      double precision budget_weight(1:Ncontrib_mx)
      double precision lambda_gas_lo(1:Nb_gas_mx)
      double precision lambda_gas_hi(1:Nb_gas_mx)
      double precision w_quad(1:Nq_mx)
c     I/O
      double precision sum_flux_up(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision sum_flux_dn(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision sum_flux_net(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision sum_flux_toa(1:Ntheta_mx,1:Nphi_mx,
     &     1:Ntheta_mx,1:Nphi_mx,1:Nb_gas_mx)
      double precision sum_budget(0:Nz_mx+1,1:Ntheta_mx,1:Nphi_mx)
      double precision sum2_flux_up(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision sum2_flux_dn(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision sum2_flux_net(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision sum2_flux_toa(1:Ntheta_mx,1:Nphi_mx,
     &     1:Ntheta_mx,1:Nphi_mx,1:Nb_gas_mx)
      double precision sum2_budget(0:Nz_mx+1,1:Ntheta_mx,1:Nphi_mx)
c     temp
      integer strlen,ir,itheta,iphi,u_itheta,u_iphi,band,quad,i
      double precision dlambda
      double precision contrib
      character*100 label
      label='subroutine update_solar_results'
      
c     Debug
      if (db.eq.1) then
         write(*,*) label(1:strlen(label)),' :in'
      endif
c     Debug
c     upward fluxes
      do i=1,Ncontrib_flux_up
         ir=flux_idx_up(i,1)
         itheta=flux_idx_up(i,2)
         iphi=flux_idx_up(i,3)
         band=flux_idx_up(i,4)
         quad=flux_idx_up(i,5)
         dlambda=lambda_gas_hi(band)-lambda_gas_lo(band)
         contrib=flux_contrib_up(i)*flux_weight_up(i) ! W/m
     &        *dlambda*w_quad(quad) ! W
         sum_flux_up(ir,itheta,iphi)=sum_flux_up(ir,itheta,iphi)+
     &        contrib ! W
         sum2_flux_up(ir,itheta,iphi)=sum2_flux_up(ir,itheta,iphi)+
     &        contrib**2.0D+0 ! W
      enddo ! i
c     downward fluxes
      do i=1,Ncontrib_flux_dn
         ir=flux_idx_dn(i,1)
         itheta=flux_idx_dn(i,2)
         iphi=flux_idx_dn(i,3)
         band=flux_idx_dn(i,4)
         quad=flux_idx_dn(i,5)
         dlambda=lambda_gas_hi(band)-lambda_gas_lo(band)
         contrib=flux_contrib_dn(i)*flux_weight_dn(i) ! W/m
     &        *dlambda*w_quad(quad) ! W
c     Debug
c         write(*,*) 'band=',band
c         write(*,*) 'flux_contrib_dn(',i,')=',flux_contrib_dn(i)
c         write(*,*) 'flux_weight_dn(',i,')=',flux_weight_dn(i)
c         write(*,*) 'dlambda(',band,')=',dlambda
c         write(*,*) 'w_quad(',quad,')=',w_quad(quad)
c         write(*,*) 'contrib=',contrib
c     Debug
         sum_flux_dn(ir,itheta,iphi)=sum_flux_dn(ir,itheta,iphi)+
     &        contrib ! W
         sum2_flux_dn(ir,itheta,iphi)=sum2_flux_dn(ir,itheta,iphi)+
     &        contrib**2.0D+0 ! W
      enddo ! i
c     net fluxes
      do i=1,Ncontrib_flux_net
         ir=flux_idx_net(i,1)
         itheta=flux_idx_net(i,2)
         iphi=flux_idx_net(i,3)
         band=flux_idx_net(i,4)
         quad=flux_idx_net(i,5)
         dlambda=lambda_gas_hi(band)-lambda_gas_lo(band)
         contrib=flux_contrib_net(i)*flux_weight_net(i) ! W/m
     &        *dlambda*w_quad(quad) ! W
         sum_flux_net(ir,itheta,iphi)=sum_flux_net(ir,itheta,iphi)+
     &        contrib ! W
         sum2_flux_net(ir,itheta,iphi)=sum2_flux_net(ir,itheta,iphi)+
     &        contrib**2.0D+0 ! W
      enddo ! i
      if (toa_ad.eq.1) then
c     TOA fluxes
         do i=1,Ncontrib_flux_toa
            itheta=flux_idx_toa(i,1)
            iphi=flux_idx_toa(i,2)
            u_itheta=flux_idx_toa(i,3)
            u_iphi=flux_idx_toa(i,4)
            band=flux_idx_toa(i,5)
            quad=flux_idx_toa(i,6)
            dlambda=lambda_gas_hi(band)-lambda_gas_lo(band)
            contrib=flux_contrib_toa(i)*flux_weight_toa(i) ! W/m
     &           *dlambda*w_quad(quad) ! W
            sum_flux_toa(itheta,iphi,u_itheta,u_iphi,band)=
     &           sum_flux_toa(itheta,iphi,u_itheta,u_iphi,band)+
     &           contrib        ! W
            sum2_flux_toa(itheta,iphi,u_itheta,u_iphi,band)=
     &           sum2_flux_toa(itheta,iphi,u_itheta,u_iphi,band)+
     &           contrib**2.0D+0 ! W
         enddo                  ! i
      endif ! toa_ad=1
c     budgets
      do i=1,Ncontrib_budget
         ir=budget_idx(i,1)
         itheta=budget_idx(i,2)
         iphi=budget_idx(i,3)
         band=budget_idx(i,4)
         quad=budget_idx(i,5)
         dlambda=lambda_gas_hi(band)-lambda_gas_lo(band)
         contrib=budget_contrib(i)*budget_weight(i)
     &        *dlambda*w_quad(quad) ! W
         sum_budget(ir,itheta,iphi)=sum_budget(ir,itheta,iphi)+
     &        contrib ! W
         sum2_budget(ir,itheta,iphi)=sum2_budget(ir,itheta,iphi)+
     &        contrib**2.0D+0 ! W
      enddo ! i
c     Debug
      if (db.eq.1) then
         write(*,*) label(1:strlen(label)),' :out'
      endif
c     Debug

      return
      end





      subroutine init_sums(Nr,Ntheta,Nphi,Nb_gas,
     &     toa_ad,
     &     sum_flux_up,
     &     sum_flux_dn,
     &     sum_flux_net,
     &     sum_flux_toa,
     &     sum_budget,
     &     sum2_flux_up,
     &     sum2_flux_dn,
     &     sum2_flux_net,
     &     sum2_flux_toa,
     &     sum2_budget)
      implicit none
c
c     Purpose: to initialize arrays for solar computation
c
      include 'max.inc'
      integer Nr(1:Ntheta_mx,1:Nphi_mx),Ntheta,Nphi,Nb_gas
      integer toa_ad
      double precision sum_flux_up(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx) ! W
      double precision sum_flux_dn(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx) ! W
      double precision sum_flux_net(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx) ! W
      double precision sum_flux_toa(1:Ntheta_mx,1:Nphi_mx,
     &     1:Ntheta_mx,1:Nphi_mx,1:Nb_gas_mx) ! W
      double precision sum_budget(0:Nz_mx+1,1:Ntheta_mx,1:Nphi_mx) ! W
      double precision sum2_flux_up(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx) ! W
      double precision sum2_flux_dn(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx) ! W
      double precision sum2_flux_net(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx) ! W
      double precision sum2_flux_toa(1:Ntheta_mx,1:Nphi_mx,
     &     1:Ntheta_mx,1:Nphi_mx,1:Nb_gas_mx) ! W
      double precision sum2_budget(0:Nz_mx+1,1:Ntheta_mx,1:Nphi_mx) ! W
      integer ir,itheta,iphi,u_itheta,u_iphi,band

      do itheta=1,Ntheta
         do iphi=1,Nphi
            do ir=0,Nr(itheta,iphi)
               sum_flux_up(ir,itheta,iphi)=0.0D+0
               sum_flux_dn(ir,itheta,iphi)=0.0D+0
               sum_flux_net(ir,itheta,iphi)=0.0D+0
               sum_budget(ir,itheta,iphi)=0.0D+0
               sum2_flux_up(ir,itheta,iphi)=0.0D+0
               sum2_flux_dn(ir,itheta,iphi)=0.0D+0
               sum2_flux_net(ir,itheta,iphi)=0.0D+0
               sum2_budget(ir,itheta,iphi)=0.0D+0
            enddo               ! ir
         enddo                  ! iphi
      enddo                     ! itheta
      do itheta=1,Ntheta
         do iphi=1,Nphi
            sum_budget(Nr(itheta,iphi)+1,itheta,iphi)=0.0D+0
            sum2_budget(Nr(itheta,iphi)+1,itheta,iphi)=0.0D+0
         enddo ! iphi
      enddo ! itheta
      if (toa_ad.eq.1) then
         do itheta=1,Ntheta
            do iphi=1,Nphi
               do u_itheta=1,Ntheta
                  do u_iphi=1,Nphi
                     do band=1,Nb_gas
                        sum_flux_toa(itheta,iphi,
     &                       u_itheta,u_iphi,band)=0.0D+0
                        sum2_flux_toa(itheta,iphi,
     &                       u_itheta,u_iphi,band)=0.0D+0
                     enddo      ! band
                  enddo         ! u_iphi
               enddo            ! u_itheta
            enddo               ! iphi
         enddo                  ! itheta
      endif ! toa_ad=1
      
      return
      end




      subroutine update_sums(Nr,Ntheta,Nphi,Nb_gas,toa_ad,
     &     psum_flux_up,psum_flux_dn,psum_flux_net,
     &     psum_flux_toa,psum_budget,
     &     psum2_flux_up,psum2_flux_dn,psum2_flux_net,
     &     psum2_flux_toa,psum2_budget,
     &     sum_flux_up,sum_flux_dn,sum_flux_net,
     &     sum_flux_toa,sum_budget,
     &     sum2_flux_up,sum2_flux_dn,sum2_flux_net,
     &     sum2_flux_toa,sum2_budget)
      implicit none
c
c     Purpose: to update sum arrays for solar computation
c
      include 'max.inc'
c     I/O
      integer Nr(1:Ntheta_mx,1:Nphi_mx),Ntheta,Nphi,Nb_gas
      integer toa_ad
      double precision psum_flux_up(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision psum_flux_dn(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision psum_flux_net(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision psum_flux_toa(1:Ntheta_mx,1:Nphi_mx,
     &     1:Ntheta_mx,1:Nphi_mx,1:Nb_gas_mx)
      double precision psum_budget(0:Nz_mx+1,1:Ntheta_mx,1:Nphi_mx)
      double precision psum2_flux_up(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision psum2_flux_dn(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision psum2_flux_net(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision psum2_flux_toa(1:Ntheta_mx,1:Nphi_mx,
     &     1:Ntheta_mx,1:Nphi_mx,1:Nb_gas_mx)
      double precision psum2_budget(0:Nz_mx+1,1:Ntheta_mx,1:Nphi_mx)
      double precision sum_flux_up(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision sum_flux_dn(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision sum_flux_net(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision sum_flux_toa(1:Ntheta_mx,1:Nphi_mx,
     &     1:Ntheta_mx,1:Nphi_mx,1:Nb_gas_mx)
      double precision sum_budget(0:Nz_mx+1,1:Ntheta_mx,1:Nphi_mx)
      double precision sum2_flux_up(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision sum2_flux_dn(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision sum2_flux_net(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision sum2_flux_toa(1:Ntheta_mx,1:Nphi_mx,
     &     1:Ntheta_mx,1:Nphi_mx,1:Nb_gas_mx)
      double precision sum2_budget(0:Nz_mx+1,1:Ntheta_mx,1:Nphi_mx)
c     temp
      integer ir,itheta,iphi,u_itheta,u_iphi,band

      do itheta=1,Ntheta
         do iphi=1,Nphi

            do ir=0,Nr(itheta,iphi)
               sum_flux_up(ir,itheta,iphi)=
     &              sum_flux_up(ir,itheta,iphi)+
     &              psum_flux_up(ir,itheta,iphi)
               sum_flux_dn(ir,itheta,iphi)=
     &              sum_flux_dn(ir,itheta,iphi)+
     &              psum_flux_dn(ir,itheta,iphi)
               sum_flux_net(ir,itheta,iphi)=
     &              sum_flux_net(ir,itheta,iphi)+
     &              psum_flux_net(ir,itheta,iphi)
               sum_budget(ir,itheta,iphi)=
     &              sum_budget(ir,itheta,iphi)+
     &              psum_budget(ir,itheta,iphi)
               sum2_flux_up(ir,itheta,iphi)=
     &              sum2_flux_up(ir,itheta,iphi)+
     &              psum2_flux_up(ir,itheta,iphi)
               sum2_flux_dn(ir,itheta,iphi)=
     &              sum2_flux_dn(ir,itheta,iphi)+
     &              psum2_flux_dn(ir,itheta,iphi)
               sum2_flux_net(ir,itheta,iphi)=
     &              sum2_flux_net(ir,itheta,iphi)+
     &              psum2_flux_net(ir,itheta,iphi)
               sum2_budget(ir,itheta,iphi)=
     &              sum2_budget(ir,itheta,iphi)+
     &              psum2_budget(ir,itheta,iphi)
            enddo               ! ir

            if (toa_ad.eq.1) then
               do u_itheta=1,Ntheta
                  do u_iphi=1,Nphi
                     do band=1,Nb_gas
                        sum_flux_toa(itheta,iphi,u_itheta,u_iphi,band)=
     &                       sum_flux_toa(itheta,iphi,
     &                       u_itheta,u_iphi,band)+
     &                       psum_flux_toa(itheta,iphi,
     &                       u_itheta,u_iphi,band)
                        sum2_flux_toa(itheta,iphi,u_itheta,u_iphi,band)=
     &                       sum2_flux_toa(itheta,iphi,
     &                       u_itheta,u_iphi,band)+
     &                       psum2_flux_toa(itheta,iphi,
     &                       u_itheta,u_iphi,band)
                     enddo      ! band
                  enddo         ! u_iphi
               enddo            ! u_itheta
            endif               ! toa_ad=1

         enddo                  ! iphi
      enddo                     ! itheta

      return
      end




      subroutine solar_emission(band,lambda_gas_lo,lambda_gas_hi,
     &     Star_temperature,Star_radius,distance,solar_power)
      implicit none
      include 'max.inc'
      include 'param.inc'
c     
c     Purpose: to provide the power emitted by the star that reaches the top of the atmosphere,
c     in a given narrowband spectral interval.
c
c     Inputs:
c       + band: index of the narrowband spectral interval
c       + lambda_gas_lo: lower limits of gas spectral data narrowband intervals (m)
c       + lambda_gas_hi: higher limits of gas spectral data narrowband intervals (m)
c       + Star_temperature: temperature of the star (K)
c       + Star_radius: raidus of the star (m)
c       + distance: distance from star to planet (m)
c
c     Output:
c       + solar_power: power emitted by the star, that reaches the top of the atmosphere (W/m/m)
c

c     Input
      integer band
      double precision lambda_gas_lo(1:Nb_gas_mx)
      double precision lambda_gas_hi(1:Nb_gas_mx)
      double precision Star_temperature,Star_radius,distance
c     Output
      double precision solar_power
c     temp
      integer strlen
      double precision planck_int_wavlng
      double precision Star_surface
      double precision orbit_surface
      double precision B,nu_lo,nu_hi
      double precision dlambda
      character*100 label
      label='subroutine solar_emission'

      if (lambda_gas_hi(band).lt.lambda_gas_lo(band)) then
         write(*,*) 'Error from ',label(1:strlen(label)),' :'
         write(*,*) 'lambda_gas_lo(',band,')=',lambda_gas_lo(band)
         write(*,*) 'lambda_gas_hi(',band,')=',lambda_gas_hi(band)
         stop
      endif

      dlambda=lambda_gas_hi(band)-lambda_gas_lo(band)
c     B is a Planck intensity, in W/m/sr (already integrated over the spectral interval)
      B=planck_int_wavlng(lambda_gas_lo(band),lambda_gas_hi(band),
     &     Star_temperature)    ! W/m/sr
      Star_surface=4.0D+0*pi*Star_radius**2.0D+0 ! m
      orbit_surface=4.0D+0*pi*distance**2.0D+0 ! m
      solar_power=pi*B*Star_surface/orbit_surface/dlambda ! W/m/m

      return
      end
